`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name:    sata_core
// Module Name:    cominit_detector 
// Project Name: 
// Target Devices: Virtex-4
// Tool versions: ISE 14.2
// Description: Detects a COMINIT OOB signal from the disk. Asserts _detected_ when 
// received, and waits for an ack before de-asserting it.
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module cominit_detector # (
//Parameters that determine the burst and inter-burst times (106ns, 320ns)
//Calculate based on clock speed, e.g. for 100Mhz clock:
//106e-9 * 100e6 = 10.6 -> round to 11
	parameter COUNT_106NS 		= 11,
	parameter COUNT_320NS 		= 32,
//Tolerances for detection around the ideal values (106ns and 320ns)
	parameter BURST_TOLERANCE 	= 6, //Default: 50ns - 170ns
	parameter GAP_TOLERANCE 	= 12 //Default: 200ns - 440ns
)(
    input clk,
    input reset,
    input rxsigdet,
    output reg detected,
    input ack
    );

parameter BURST_LOW = COUNT_106NS - BURST_TOLERANCE,
			 BURST_HIGH = COUNT_106NS + BURST_TOLERANCE;
parameter GAP_LOW = COUNT_320NS - GAP_TOLERANCE,
			 GAP_HIGH = COUNT_320NS + GAP_TOLERANCE;
          
//Register the rxsigdet input
reg rxsigdet_r;
always @ (posedge clk) begin
   rxsigdet_r <= rxsigdet;
end

	 
//States
parameter [1:0]
	s_idle = 2'b00,
	s_burst = 2'b01,
	s_gap = 2'b10,
	s_detected = 2'b11;
	
reg [1:0] state, state_next;
reg detected_next;
reg [2:0] burst_counter, burst_counter_next;
reg [5:0] time_counter, time_counter_next;

always @ (*)
begin //Comb FSM
	//defaults
	state_next = state;
	detected_next = detected;
	burst_counter_next = burst_counter;
	time_counter_next = time_counter;
	
	case (state)
	//Idle state: waiting for a signal
		s_idle:
			begin
				detected_next = 1'b0;
				burst_counter_next = 3'b0;
				time_counter_next = 6'b0;
				if (rxsigdet_r == 1'b0) //detected an OOB signal
					state_next = s_burst;
				else
					state_next = s_idle;
			end
	//Burst state: wait until end of burst. Check length of the burst. Keep track of the number of bursts.
	//After 6 bursts, go to the detection state.
		s_burst:
			begin
				time_counter_next = time_counter + 1;
				//timeout condition
				if (time_counter >= BURST_HIGH) //17: 170ns
					state_next = s_idle;
					
				if (rxsigdet_r == 1'b1) //end of burst
					if (time_counter >= BURST_LOW) begin//5: 50ns, well-formed burst
						if (burst_counter == 3'b101) begin //This is the sixth burst
							state_next = s_detected;
						end
						else begin 
							state_next = s_gap;
							burst_counter_next = burst_counter + 1;
							time_counter_next = 6'b0; //reset the counter
						end
					end
					else
						state_next = s_idle;
			end
			
	//Gap state: Check length of time between bursts
	//Detects on 200ns to 440ns
		s_gap:
			begin
				time_counter_next = time_counter + 1;
				//timeout condition
				if (time_counter >= GAP_HIGH) //45: 450ns
					state_next = s_idle;
					
				if (rxsigdet_r == 1'b0) begin //start of a new burst
					if (time_counter >= GAP_LOW) begin //20: 200ns
						state_next = s_burst;
						time_counter_next = 6'b0;
					end
					else
						state_next = s_idle;
				end
			end
			
	//Detected state: Asserts _detected_, waits for acknowledgment
		s_detected:
			begin
				detected_next = 1'b1;
				if (ack == 1'b1)
					state_next = s_idle;
			end
		
		default: state_next = s_idle;
	
	endcase
	
end


always @ (posedge clk or posedge reset)
begin //Seq FSM
	if (reset) begin
		state <= s_idle;
		detected <= 1'b0;
		burst_counter <= 3'b0;
		time_counter <= 6'b0;
	end
	else begin
		state <= state_next;
		detected <= detected_next;
		burst_counter <= burst_counter_next;
		time_counter <= time_counter_next;
	end
end

endmodule
