-- Copyright (C) 2012
-- Ashwin A. Mendon
--
-- This file is part of SATA2 core.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.  

----------------------------------------------------------------------------------------
-- ENTITY: command_layer 
-- Version: 2.0
-- Author:  Cory Gorman
-- Changes:
--    Switches to the numeric_std library.
--    Adds replay buffer functionality to gracefully recover from transmission errors.
-- Version: 1.0
-- Author:  Ashwin Mendon 
-- Description: This sub-module implements the Command Layer of the SATA Protocol
--              The User Command parameters such as: cmd_type, sector_address, sector_count
--              are encoded into a command FIS according to the ATA format and passed to
--              the Transport Layer.                   
--              
-- PORTS: 
-----------------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

entity command_layer is
  generic(
    CHIPSCOPE           : boolean := true;
    USE_REPLAY_BUFFER   : boolean := true
       );
  port(
     -- Clock and Reset Signals
    clk                   : in  std_logic;
    sw_reset              : in  std_logic;
    -- ChipScope ILA / Trigger Signals
    cmd_layer_ila_control  : inout  std_logic_vector(35 downto 0);
    ---------------------------------------
    -- Signals from/to User Logic
    new_cmd               : in  std_logic;
    cmd_done              : out std_logic;
    cmd_type	          : in  std_logic_vector(1 downto 0);
    sector_count          : in  std_logic_vector(31 downto 0);
    sector_addr           : in  std_logic_vector(31 downto 0);
    user_din              : in  std_logic_vector(31 downto 0); 
    user_din_re_out       : out std_logic; 
    user_dout             : out std_logic_vector(31 downto 0); 
    user_dout_re          : in std_logic; 
    user_fifo_empty       : in std_logic; 
    user_fifo_full        : in std_logic; 
    sector_timer_out      : out std_logic_vector(47 downto 0);
    -- Signals from/to Link Layer
    write_fifo_full       : in std_logic;
    ll_ready_for_cmd      : in std_logic;
    ll_cmd_start	  : out std_logic;
    ll_cmd_type	          : out std_logic_vector(1 downto 0);
    ll_dout               : out std_logic_vector(31 downto 0);
    ll_dout_we            : out std_logic;
    ll_din                : in  std_logic_vector(31 downto 0);
    ll_din_re             : out  std_logic;
    sector_count_out      : out  std_logic_vector(31 downto 0);
	 
	 r_err_in				  : in std_logic;
    tx_ok_in              : in std_logic;
    tx_err_in             : in std_logic;
    
    link_error            : in std_logic;
    link_error_ack        : out std_logic;
    sata_low_reset        : out std_logic
      );
end command_layer;

-------------------------------------------------------------------------------
-- ARCHITECTURE
-------------------------------------------------------------------------------
architecture BEHAV of command_layer is

 -------------------------------------------------------------------------------
 -- COMMAND LAYER
 -------------------------------------------------------------------------------
  constant READ_DMA         : std_logic_vector(7 downto 0) := x"25";
  constant WRITE_DMA        : std_logic_vector(7 downto 0) := x"35";
  constant REG_FIS_VALUE    : std_logic_vector(7 downto 0) := x"27";
  constant DATA_FIS_VALUE   : std_logic_vector(7 downto 0) := x"46";
  constant DEVICE_REG       : std_logic_vector(7 downto 0) := x"E0";
  constant FEATURES         : std_logic_vector(7 downto 0) := x"00";
  constant READ_DMA_CMD     : std_logic_vector(1 downto 0) := "01";
  constant WRITE_DMA_CMD    : std_logic_vector(1 downto 0) := "10";
  constant DATA_FIS_HEADER  : std_logic_vector(31 downto 0) := x"00000046";
  constant NDWORDS_PER_DATA_FIS : unsigned(15 downto 0) := to_unsigned(2048, 16);--128*16    
  constant SECTOR_NDWORDS     : integer := 128;  -- 128 DWORDS / 512 Byte Sector    

  component cmd_layer_ila
    port (
      control : inout std_logic_vector(35 downto 0);
      clk     : in std_logic;
      trig0   : in std_logic_vector(3  downto 0);
      trig1   : in std_logic_vector(31 downto 0);
      trig2   : in std_logic_vector(31 downto 0);
      trig3   : in std_logic_vector(31 downto 0);
      trig4   : in std_logic_vector(31 downto 0);
      trig5   : in std_logic_vector(1 downto 0);
      trig6   : in std_logic_vector(1 downto 0);
      trig7   : in std_logic_vector(31 downto 0);
      trig8   : in std_logic_vector(31 downto 0);
      trig9   : in std_logic_vector(23 downto 0);
      trig10  : in std_logic_vector(15 downto 0);
      trig11  : in std_logic_vector(11 downto 0);
      trig12  : in std_logic_vector(15 downto 0);
      trig13  : in std_logic_vector(31 downto 0)
   );
  end component;
  
   component replay_buffer
   port (
      clk: IN std_logic;
      rst: IN std_logic;
      rd_en: IN std_logic;
      din: IN std_logic_VECTOR(31 downto 0);
      wr_en: IN std_logic;
      dout: OUT std_logic_VECTOR(31 downto 0);
      empty: OUT std_logic;
      full: OUT std_logic
   );
   end component;


 -----------------------------------------------------------------------------
  -- Finite State Machine Declaration (curr and next states)
 -----------------------------------------------------------------------------
  type COMMAND_FSM_TYPE is (wait_for_cmd, prepare_transfer, build_REG_FIS, send_REG_FIS_DW1, 
                           send_REG_FIS_DW2, send_REG_FIS_DW3, send_REG_FIS_DW4, send_REG_FIS_DW5,
                           send_DATA_FIS_HEADER, send_write_data, wait_for_status, send_cmd_start, 
                           wait_for_cmd_start, wait_for_cmd_done, dead                           
                     );
  signal command_fsm_curr, command_fsm_next : COMMAND_FSM_TYPE := wait_for_cmd; 
  signal command_fsm_value                  : std_logic_vector (0 to 3);
  
  signal ll_cmd_start_next                 : std_logic;
  signal ll_cmd_start_out                  : std_logic;
  signal cmd_done_next                  : std_logic;
  signal cmd_done_out                   : std_logic;
  signal read_fifo_empty                : std_logic;
  signal ll_dout_next                   : std_logic_vector(0 to 31); 
  signal ll_dout_we_next                : std_logic; 
  signal ll_dout_out                    : std_logic_vector(0 to 31); 
  signal ll_dout_we_out                 : std_logic; 
  signal ll_cmd_type_next               : std_logic_vector(0 to 1); 
  signal ll_cmd_type_out                : std_logic_vector(0 to 1); 
  signal dword_count                    : unsigned(0 to 15); 
  signal dword_count_next               : unsigned(0 to 15); 
  signal write_data_count               : unsigned(0 to 31); 
  signal write_data_count_next          : unsigned(0 to 31); 
  signal user_din_re                    : std_logic; 
  signal sector_count_int               : integer; 

  --- ILA signals ----
  signal user_dout_ila                  : std_logic_vector(0 to 31); 
  signal ll_din_re_ila                  : std_logic; 

  --- Timer ----
  signal sector_timer                   : unsigned(47 downto 0);
  
  signal error_counter : unsigned(31 downto 0);
  
  --- Replay Buffer ---
  signal replay_reset              : std_logic;
  signal replay_re               : std_logic;
  signal replay_we               : std_logic;
  signal replay_empty            : std_logic;
  signal replay_full             : std_logic;
  signal replay_dout             : std_logic_vector(31 downto 0);
  signal replay_din              : std_logic_vector(31 downto 0);
  signal replay_flag             : std_logic;
  signal replay_flag_next        : std_logic;
  signal replay_flush            : std_logic;
  signal replay_flush_next       : std_logic;
  -- Signals for calculating new values for the replay buffer.
  signal curr_sector_count, curr_sector_count_next : unsigned(15 downto 0);
  signal curr_sector_addr, curr_sector_addr_next : unsigned(23 downto 0);
  signal n_successes, n_successes_next : unsigned(15 downto 0);
  signal last_fis_flag, last_fis_flag_next : std_logic;
  --Error detection
  signal link_error_ack_i, link_error_ack_next  : std_logic;
  signal sata_low_reset_i, sata_low_reset_next  : std_logic;

  type reg_fis_type is
    record
      FIS_type      : std_logic_vector(7 downto 0);
      pad_8         : std_logic_vector(7 downto 0);
      command       : std_logic_vector(7 downto 0);
      features      : std_logic_vector(7 downto 0);
      LBA           : std_logic_vector(23 downto 0);
      device        : std_logic_vector(7 downto 0);
      LBA_exp       : std_logic_vector(23 downto 0);
      features_exp  : std_logic_vector(7 downto 0);
      sector_count  : std_logic_vector(15 downto 0);
      pad_16        : std_logic_vector(15 downto 0);
      pad_32        : std_logic_vector(31 downto 0);
    end record; 

  signal reg_fis   : reg_fis_type;
  signal reg_fis_next   : reg_fis_type;

-------------------------------------------------------------------------------
-- BEGIN
-------------------------------------------------------------------------------
begin

-------------------------------------------------------------------------------
-- LINK LAYER
-------------------------------------------------------------------------------
  -----------------------------------------------------------------------------
  -- PROCESS: COMMAND_FSM_VALUE_PROC
  -- PURPOSE: ChipScope State Indicator Signal
  -----------------------------------------------------------------------------
  COMMAND_FSM_VALUE_PROC : process (command_fsm_curr) is
  begin
    case (command_fsm_curr) is
      when wait_for_cmd       => command_fsm_value <= x"0";
      when prepare_transfer   => command_fsm_value <= x"F";
      when build_REG_FIS      => command_fsm_value <= x"1";
      when send_REG_FIS_DW1   => command_fsm_value <= x"2";
      when send_REG_FIS_DW2   => command_fsm_value <= x"3";
      when send_REG_FIS_DW3   => command_fsm_value <= x"4";
      when send_REG_FIS_DW4   => command_fsm_value <= x"5";
      when send_REG_FIS_DW5   => command_fsm_value <= x"6";
      when send_DATA_FIS_HEADER => command_fsm_value <= x"7";
      when send_write_data    => command_fsm_value <= x"8";
      when wait_for_status    => command_fsm_value <= x"E";
      when send_cmd_start     => command_fsm_value <= x"9";
      when wait_for_cmd_start => command_fsm_value <= x"A";
      when wait_for_cmd_done  => command_fsm_value <= x"B";
      when dead               => command_fsm_value <= x"C";
      when others             => command_fsm_value <= x"D";
    end case;
  end process COMMAND_FSM_VALUE_PROC;
  
  -----------------------------------------------------------------------------
  -- PROCESS: COMMAND_FSM_STATE_PROC
  -- PURPOSE: Registering Signals and Next State
  -----------------------------------------------------------------------------
  COMMAND_FSM_STATE_PROC: process (clk)
  begin
    if ((clk'event) and (clk = '1')) then
      if (sw_reset = '1') then
        --Initializing internal signals
        command_fsm_curr       <= wait_for_cmd;
        cmd_done_out           <= '0';
        ll_cmd_start_out       <= '0';
        ll_dout_we_out         <= '0'; 
        ll_dout_out            <= (others => '0'); 
        ll_cmd_type_out        <= (others => '0');
        write_data_count       <= (others => '0');
        dword_count            <= (others => '0');
        reg_fis.FIS_type       <= (others => '0');
        reg_fis.pad_8          <= (others => '0');
        reg_fis.command        <= (others => '0');
        reg_fis.features       <= (others => '0');
        reg_fis.LBA            <= (others => '0');
        reg_fis.device         <= (others => '0');
        reg_fis.LBA_exp        <= (others => '0');
        reg_fis.features_exp   <= (others => '0');
        reg_fis.sector_count   <= (others => '0');
        reg_fis.pad_16         <= (others => '0');
        reg_fis.pad_32         <= (others => '0');
        replay_flush           <= '0';
        replay_flag            <= '0';
        curr_sector_count      <= (others => '0');
        curr_sector_addr       <= (others => '0');
        n_successes            <= (others => '0');
        last_fis_flag          <= '0';
        link_error_ack_i       <= '0';
        sata_low_reset_i       <= '0';
      else
        -- Register all Current Signals to their _next Signals
        command_fsm_curr       <= command_fsm_next;
        cmd_done_out           <= cmd_done_next;
        ll_cmd_start_out       <= ll_cmd_start_next;
        ll_dout_we_out         <= ll_dout_we_next; 
        ll_dout_out            <= ll_dout_next; 
        ll_cmd_type_out        <= ll_cmd_type_next;
        dword_count            <= dword_count_next;
        write_data_count       <= write_data_count_next;
        reg_fis.FIS_type       <= reg_fis_next.FIS_type ;
        reg_fis.pad_8          <= reg_fis_next.pad_8;
        reg_fis.command        <= reg_fis_next.command;
        reg_fis.features       <= reg_fis_next.features;
        reg_fis.LBA            <= reg_fis_next.LBA;
        reg_fis.device         <= reg_fis_next.device;
        reg_fis.LBA_exp        <= reg_fis_next.LBA_exp;
        reg_fis.features_exp   <= reg_fis_next.features_exp;
        reg_fis.sector_count   <= reg_fis_next.sector_count;
        reg_fis.pad_16         <= reg_fis_next.pad_16;
        reg_fis.pad_32         <= reg_fis_next.pad_32;
        replay_flush           <= replay_flush_next;
        replay_flag            <= replay_flag_next;
        curr_sector_count      <= curr_sector_count_next;
        curr_sector_addr       <= curr_sector_addr_next;
        n_successes            <= n_successes_next;
        last_fis_flag          <= last_fis_flag_next;
        link_error_ack_i       <= link_error_ack_next;
        sata_low_reset_i       <= sata_low_reset_next;
      end if;
    end if;
  end process COMMAND_FSM_STATE_PROC;

  -----------------------------------------------------------------------------
  -- PROCESS: COMMAND_FSM_LOGIC_PROC 
  -- PURPOSE: Registering Signals and Next State
  -----------------------------------------------------------------------------
  COMMAND_FSM_LOGIC_PROC : process (command_fsm_curr, new_cmd, cmd_type, 
                                   ll_cmd_start_out, ll_dout_we_out,
                                   ll_dout_out, dword_count, write_data_count     
                                   ) is
  begin
    -- Register _next to current signals
    command_fsm_next          <= command_fsm_curr;
    cmd_done_next             <= cmd_done_out;
    ll_cmd_start_next         <= ll_cmd_start_out;
    ll_dout_we_next           <= ll_dout_we_out; 
    ll_dout_next              <= ll_dout_out; 
    ll_cmd_type_next          <= cmd_type;
    user_din_re               <= '0';
    dword_count_next          <= dword_count;
    write_data_count_next     <= write_data_count;
    reg_fis_next.FIS_type     <= reg_fis.FIS_type ;
    reg_fis_next.pad_8        <= reg_fis.pad_8;
    reg_fis_next.command      <= reg_fis.command;
    reg_fis_next.features     <= reg_fis.features;
    reg_fis_next.LBA          <= reg_fis.LBA;
    reg_fis_next.device       <= reg_fis.device;
    reg_fis_next.LBA_exp      <= reg_fis.LBA_exp;
    reg_fis_next.features_exp <= reg_fis.features_exp;
    reg_fis_next.sector_count <= reg_fis.sector_count;
    reg_fis_next.pad_16       <= reg_fis.pad_16;
    reg_fis_next.pad_32       <= reg_fis.pad_32;
    replay_flush_next         <= '0';
    replay_flag_next          <= replay_flag;
    curr_sector_count_next    <= curr_sector_count;
    curr_sector_addr_next     <= curr_sector_addr;
    n_successes_next          <= n_successes;
    last_fis_flag_next        <= last_fis_flag;
    link_error_ack_next       <= '0';
    sata_low_reset_next       <= '0';

    ---------------------------------------------------------------------------
    -- Finite State Machine
    ---------------------------------------------------------------------------
    case (command_fsm_curr) is
      
     -- x0
     when wait_for_cmd =>   
         cmd_done_next     <= '1';
         ll_cmd_start_next    <= '0';
         ll_dout_we_next   <= '0'; 
         ll_dout_next      <= (others => '0');
         replay_flag_next  <= '0';
         last_fis_flag_next   <= '0';
         if (new_cmd = '1') then
            cmd_done_next     <= '0';
            command_fsm_next  <= prepare_transfer;
         end if;
         
     when prepare_transfer =>
         curr_sector_count_next  <= unsigned(sector_count(15 downto 0));
         curr_sector_addr_next   <= unsigned(sector_addr(23 downto 0));
         n_successes_next        <= (others => '0');
         command_fsm_next        <= build_REG_FIS;

     -- x1
     when build_REG_FIS =>   
          reg_fis_next.FIS_type      <= REG_FIS_VALUE;
          reg_fis_next.pad_8         <= x"80";
          if (cmd_type = READ_DMA_CMD) then
             reg_fis_next.command    <= READ_DMA;
          else
             reg_fis_next.command    <= WRITE_DMA;
          end if;
          reg_fis_next.features      <= FEATURES;
          reg_fis_next.LBA           <= std_logic_vector(curr_sector_addr);
          reg_fis_next.device        <= DEVICE_REG;
          reg_fis_next.LBA_exp       <= (others => '0');
          reg_fis_next.features_exp  <= FEATURES;
          reg_fis_next.sector_count  <= std_logic_vector(curr_sector_count);
          reg_fis_next.pad_16        <= (others => '0');
          reg_fis_next.pad_32        <= (others => '0');
          command_fsm_next           <= send_REG_FIS_DW1;

     -- x2
     when send_REG_FIS_DW1 => 
          ll_dout_next         <= reg_fis.FEATURES & reg_fis.command & reg_fis.pad_8 & reg_fis.FIS_type;
          ll_dout_we_next      <= '1';
          command_fsm_next     <= send_REG_FIS_DW2;
   
     -- x3
     when send_REG_FIS_DW2 => 
          ll_dout_next         <= reg_fis.device & reg_fis.LBA;
          ll_dout_we_next      <= '1';
          command_fsm_next     <= send_REG_FIS_DW3;
  
     -- x4
     when send_REG_FIS_DW3 => 
          ll_dout_next         <= reg_fis.features_exp & reg_fis.LBA_exp;
          ll_dout_we_next      <= '1';
          command_fsm_next     <= send_REG_FIS_DW4;
     
     -- x5
     when send_REG_FIS_DW4 => 
          ll_dout_next         <= reg_fis.pad_16 & reg_fis.sector_count ;
          ll_dout_we_next      <= '1';
          command_fsm_next     <= send_REG_FIS_DW5;
     
     -- x6
     when send_REG_FIS_DW5 => 
          ll_dout_next         <= reg_fis.pad_32;
          ll_dout_we_next      <= '1';
          command_fsm_next  <= send_cmd_start;

     -- x7
     when send_DATA_FIS_HEADER =>
          if (user_fifo_full = '1') then
             ll_dout_next         <= DATA_FIS_HEADER;
             ll_dout_we_next      <= '1';
             command_fsm_next     <= send_write_data;
          end if;

     -- x8
     when send_write_data =>
          if(dword_count >= NDWORDS_PER_DATA_FIS) then
            user_din_re        <= '0';
            ll_dout_we_next    <= '0'; 
            ll_dout_next       <= (others => '0');
            dword_count_next   <= (others => '0');
            command_fsm_next   <= wait_for_status;
          elsif (write_fifo_full = '1' or user_fifo_empty = '1') then
            user_din_re        <= '0';
            ll_dout_we_next    <= '0'; 
            ll_dout_next       <= (others => '0');
          else 
            write_data_count_next <= write_data_count + 1;
            dword_count_next   <= dword_count + 1;
            if (replay_flag = '1') then
               ll_dout_next    <= replay_dout;
               replay_re       <= '1';
            else
               ll_dout_next       <= user_din;
               user_din_re        <= '1'; 
            end if;
            ll_dout_we_next    <= '1';
          end if;
          if (write_data_count = to_unsigned(SECTOR_NDWORDS*sector_count_int, 32)) then
             user_din_re        <= '0';
             ll_dout_we_next    <= '0'; 
             ll_dout_next       <= (others => '0');
             command_fsm_next   <= wait_for_status;
             last_fis_flag_next <= '1';
          end if;
          
     when wait_for_status =>
         if (tx_ok_in = '1') then
            if (last_fis_flag = '0') then
               command_fsm_next   <= send_DATA_FIS_HEADER;
               replay_flush_next <= '1';
               replay_flag_next  <= '0';
               n_successes_next  <= n_successes + 1;
            else
               command_fsm_next   <= wait_for_cmd_done;
               write_data_count_next <= (others => '0');
               dword_count_next   <= (others => '0');
            end if;
         elsif ((tx_err_in = '1') or (link_error = '1')) then
         --recalculate values, reset counts
            replay_flag_next  <= '1';
            --Prepare a new transfer
            if ((n_successes > 0) and (curr_sector_count > (n_successes*16))) then
               curr_sector_count_next  <= curr_sector_count - (n_successes sll 4);
               curr_sector_addr_next   <= curr_sector_addr + (n_successes sll 4);
            end if;
            n_successes_next   <= (others => '0');
            write_data_count_next <= (others => '0');
            command_fsm_next   <= build_REG_FIS;
            if (link_error = '1') then
               link_error_ack_next <= '1';
               sata_low_reset_next <= '1';
            end if;
         end if;
 
     -- x9
     when send_cmd_start =>
          ll_dout_we_next   <= '0'; 
          ll_dout_next      <= (others => '0'); 
          if (ll_ready_for_cmd = '1') then
            ll_cmd_start_next           <= '1';
            command_fsm_next         <= wait_for_cmd_start;
          elsif (link_error = '1') then
            command_fsm_next   <= build_REG_FIS;
            link_error_ack_next <= '1';
            sata_low_reset_next <= '1';
          end if;

     -- xA
      when wait_for_cmd_start =>
         ll_cmd_start_next           <= '0';
         if (ll_ready_for_cmd = '0') then
            --Check to see if register FIS was sent OK
            if (tx_ok_in = '1') then
               if (cmd_type = READ_DMA_CMD) then
                  command_fsm_next   <= wait_for_cmd_done;
               else
                  command_fsm_next   <= send_DATA_FIS_HEADER;
               end if;
            --If there is an error, resend the register FIS
            elsif (tx_err_in = '1') then
               command_fsm_next     <= build_REG_FIS;
            end if;
         end if;

     -- xB
     when wait_for_cmd_done =>
          if (ll_ready_for_cmd = '1') then
             cmd_done_next     <= '1';
             command_fsm_next  <= wait_for_cmd;
             replay_flush_next <= '1';
             replay_flag_next  <= '0';
             n_successes_next  <= n_successes + 1;
          end if;
          --Error on the RegisterD2H FIS
          if (link_error = '1') then
            replay_flag_next  <= '1';
            --Prepare a new transfer
            if ((n_successes > 0) and (curr_sector_count > (n_successes*16))) then
               curr_sector_count_next  <= curr_sector_count - (n_successes sll 4);
               curr_sector_addr_next   <= curr_sector_addr + (n_successes sll 4);
            end if;
            n_successes_next   <= (others => '0');
            write_data_count_next <= (others => '0');
            command_fsm_next   <= build_REG_FIS;
            link_error_ack_next <= '1';
            sata_low_reset_next <= '1';
         end if;
 
     -- xC
     when dead =>   
         command_fsm_next  <= dead;

     -- xD
     when others =>   
         command_fsm_next  <= dead;

   end case;
 end process COMMAND_FSM_LOGIC_PROC;

  cmd_done         <= cmd_done_out;
  ll_cmd_start     <= ll_cmd_start_out;
  ll_cmd_type      <= ll_cmd_type_out;

  user_din_re_out  <= user_din_re;
  user_dout_ila    <= ll_din;
  user_dout        <= user_dout_ila;
  ll_dout          <= ll_dout_out;
  ll_dout_we       <= ll_dout_we_out;
  ll_din_re_ila    <= user_dout_re;
  ll_din_re        <= ll_din_re_ila;

  sector_count_int <= to_integer(curr_sector_count);
  link_error_ack   <= link_error_ack_i;
  sata_low_reset   <= sata_low_reset_i;

  -----------------------------------------------------------------------------
  -- PROCESS: TIMER PROCESS 
  -- PURPOSE: Count time to read a sector
  -----------------------------------------------------------------------------
  TIMER_PROC: process (clk)
  begin
    if ((clk'event) and (clk = '1')) then
      if (sw_reset = '1') then
        sector_timer    <= (others => '0');
      elsif (command_fsm_curr = wait_for_cmd) then
        if (new_cmd = '1') then
           sector_timer    <= (others => '0');
        else
           sector_timer    <= sector_timer;
        end if;
      else
        sector_timer    <= sector_timer + 1;
      end if;
    end if;
  end process TIMER_PROC;
  sector_timer_out        <= std_logic_vector(sector_timer);
  
  sector_count_out        <= std_logic_vector("0000000000000000" & curr_sector_count);
  
  ERROR_COUNTER_PROC: process (clk)
  begin
   if ((clk'event) and (clk = '1')) then
      if (sw_reset = '1') then
         error_counter <= (others => '0');
      elsif ((link_error_ack_i = '1') or (tx_err_in = '1')) then
         error_counter <= error_counter + 1;
      else
         error_counter <= error_counter;
      end if;
   end if;
 end process ERROR_COUNTER_PROC;
  
  ----------------------------------------------------------------
  -- Replay Buffer: Seamlessly recover from transmission errors
  ----------------------------------------------------------------
  replay_reset <= sw_reset or replay_flush;
  replay_we <= '1' when ((ll_dout_we_out = '1') and (command_fsm_curr = send_write_data)) else '0';
  replay_din <= ll_dout_out;
  
replay_gen : if (USE_REPLAY_BUFFER) generate
  
  replay_buffer_i : replay_buffer
   port map (
      clk => clk,
      rst => replay_reset,
      rd_en => replay_re,
      din => replay_din,
      wr_en => replay_we,
      dout => replay_dout,
      empty => replay_empty,
      full => replay_full
   );
end generate replay_gen;


chipscope_gen_ila : if (CHIPSCOPE) generate
  CMD_LAYER_ILA_i : cmd_layer_ila
    port map (
      control  => cmd_layer_ila_control,
      clk      => clk,
      trig0    => command_fsm_value,
      trig1    => user_din,
      trig2    => user_dout_ila,
      trig3    => ll_din,
      trig4    => ll_dout_out,
      trig5    => cmd_type,
      trig6    => ll_cmd_type_out,
      trig7    => std_logic_vector(sector_timer(47 downto 16)),
      trig8    => std_logic_vector(error_counter), --sector_addr,
      trig9    => reg_fis.LBA,
      trig10    => reg_fis.sector_count,
      trig11(0) => new_cmd,
      trig11(1) => tx_err_in, --user_din_re,
      trig11(2) => user_dout_re,
      trig11(3) => ll_ready_for_cmd,
      trig11(4) => last_fis_flag, --ll_cmd_start_out,
      trig11(5) => ll_dout_we_out, 
      trig11(6) => replay_flag, --ll_din_re_ila,
      trig11(7) => link_error, --cmd_done_out, 
      trig11(8) => replay_flush, --r_err_in,
      trig11(9) => link_error_ack_i, --write_fifo_full,
      trig11(10) => replay_full, --user_fifo_empty,
      trig11(11) => user_fifo_full,
      trig12    => std_logic_vector(dword_count),
      trig13    => std_logic_vector(write_data_count)
     ); 
end generate chipscope_gen_ila; 
 
end BEHAV;
