`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name:    sata_core
// Module Name:    oob_sequence_controller 
// Project Name: 
// Target Devices: Virtex-4
// Tool versions: ISE 14.2
// Description: Performs the OOB start-up sequence for SATA. Use link_reset to begin
// the process.
//
// If your system clock is not 100MHz, be sure to change the timing parameters for the
// OOB primitives.
//
// Dependencies: 
//
// Revision: 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////

module oob_sequence_controller(
    input sys_clk,
    input txclk,
    input rxclk,
    input reset,
    input link_reset,
    input rxsigdet,
	 input rx_locked,
    output txenoob,
    input [31:0] rxdatain,
    output reg [31:0] txdataout,
    input [3:0] rxcharisk,
    output reg [3:0] txcharisk,
	 //To and from Link Layer
	 input [31:0] txdatain,
	 input [3:0] txcharisk_in,
	 output reg link_ready,
	 output align_en_out,
	 output [3:0] stateOut,
	 input force_ready //for loopback testing
    );


//OOB Signal Generator

wire oob_send_en, oob_send_select, oob_rdy, oob_txenoob;

reg comreset_send_p, comwake_send_p, comreset_send_p_next, comwake_send_p_next;

oob_sig_gen #(
   .COUNT_106NS(11),
   .COUNT_320NS(32)
) oob_sig_gen_i (
    .clk (sys_clk),
    .reset (reset),
    .send_en (oob_send_en),
    .send_select (oob_send_select),
    .rdy (oob_rdy),
    .txenoob (oob_txenoob)
    );
	 
assign oob_send_en = comreset_send_p ^ comwake_send_p;
assign oob_send_select = ~comreset_send_p && comwake_send_p;


//OOB detectors

wire cominit_detected, comwake_detected;
reg detect_ack_p, detect_ack_p_next;

cominit_detector 
#(
   .COUNT_106NS(11),
   .COUNT_320NS(32),
   .BURST_TOLERANCE(6),
	.GAP_TOLERANCE(12)
) cominit_detector_i
	(
		.clk (sys_clk),
		.reset (reset),
		.rxsigdet (rxsigdet),
		.detected (cominit_detected),
		.ack (detect_ack_p)
	);
	
comwake_detector #(
   .COUNT_106NS(11),
   .TOLERANCE(6)
) comwake_detector_i
	(
		.clk (sys_clk),
		.reset (reset),
		.rxsigdet (rxsigdet),
		.detected (comwake_detected),
		.ack (detect_ack_p)
	);
   

	
reg oob_sig_en, oob_sig_en_next; //1: Use OOB signals. 0: Switch to in-band, data signals.


//Primitives
parameter [31:0]
	prim_align = 32'h7B4A4ABC, //D27.3, D10.2, D10.2, K28.5
	prim_sync =	32'hB5B5957C, // D21.5, D21.5, D21.4, K28.3
	prim_dial = 32'h4A4A4A4A; //D10.2, D10.2, D10.2, D10.2

//Specify K-Characters
parameter [3:0]
	prim_align_k = 4'b0001,
	prim_sync_k = 4'b0001,
	prim_dial_k = 4'b0000;
	
//Primitive detection
wire align_detected, sync_detected;
assign align_detected = (rxdatain == prim_align) && (rxcharisk == prim_align_k); //&& (rx_locked);
assign sync_detected = (rxdatain == prim_sync) && (rxcharisk == prim_sync_k);


//State Machine
parameter [3:0] 
	s_idle = 4'h0,
	s_send_comreset = 4'h1,
	s_wait_cominit = 4'h2,
	s_send_comwake = 4'h3,
	s_wait_comwake = 4'h4,
	s_wait_align = 4'h5,
	s_send_align = 4'h6,
	s_link_ready = 4'h7;


reg [3:0] state, state_next;
//number of cycles without changing state
reg [23:0] state_time_count;
reg state_time_count_rst;
reg link_ready_next;

//Data registers for the state machine
//Will be assigned to output until the link is ready
reg [31:0] txdataout_int, txdataout_int_next;
reg [3:0] txcharisk_int, txcharisk_int_next;

//Register signals in the sata clock domain
//Register inputs
reg rxsigdet_r, rx_locked_r, cominit_detected_r, comwake_detected_r, oob_rdy_r, link_reset_r;
always @ (posedge txclk) begin
   rxsigdet_r <= rxsigdet;
   rx_locked_r <= rx_locked;
   cominit_detected_r <= cominit_detected;
   comwake_detected_r <= comwake_detected;
   oob_rdy_r <= oob_rdy;
   link_reset_r <= link_reset;
end

always @ (*)
begin : comb_FSM
	//defaults
	state_next = state;
	link_ready_next = link_ready;
	oob_sig_en_next = oob_sig_en;
	txdataout_int_next = txdataout_int;
	txcharisk_int_next = txcharisk_int;
	
	//Pulses: default to 0
	comreset_send_p_next = 1'b0;
	comwake_send_p_next = 1'b0;
	detect_ack_p_next = 1'b0;
	
	case (state)
		//Idle: Wait for command to begin the OOB handshake
		s_idle: //0
			begin
				link_ready_next = 1'b0;
				detect_ack_p_next = 1'b1; //plugged in hard drive may send COMINITs, simply acknowledge them
				txdataout_int_next = 32'b0;
				if (link_reset_r)
					state_next = s_send_comreset;
			end
		
		//Comreset: Send COMRESET signal. Continue to send if link is held in reset.
		s_send_comreset: //1
			begin
				//spec says to send ALIGNs for the beginning of the OOB handshake, but doesn't really matter
				txdataout_int_next = prim_align;
				txcharisk_int_next = prim_align_k;
				
				oob_sig_en_next = 1'b1; //Start of handshake uses OOB signals
				link_ready_next = 1'b0;
				comreset_send_p_next = 1'b1;
				if (oob_rdy_r)
					state_next = s_wait_cominit;
			end
		
		//Cominit: Wait for COMINIT from the disk. If timeout, restart handshake.
		s_wait_cominit: //2
			begin
				if (cominit_detected_r) begin
					detect_ack_p_next = 1'b1;
					state_next = s_send_comwake;
               comwake_send_p_next = 1'b1;
				end
				else if (state_time_count == 24'h80E8) //880us passed with no response
					state_next = s_send_comreset;
			end
		
		//Comwake: Send COMWAKE signal. 
		s_send_comwake: //3
			begin
				//comwake_send_p_next = 1'b1;
				if (oob_rdy_r)
					state_next = s_wait_comwake;
			end
			
		//Comwake: Wait for COMWAKE from the disk. If timeout, restart handshake.	
		s_wait_comwake: //4
			begin
				if (comwake_detected_r) begin
					//link_ready_next = 1'b1;
					oob_sig_en_next = 1'b0; //Switch to data signals
					detect_ack_p_next = 1'b1;
					state_next = s_wait_align;
				end
				else if (state_time_count == 24'h80E8) //880us passed with no response
					state_next = s_send_comreset;
			end
		
		//Align: Speed negotiation. Look for ALIGN primitive from the disk. Meanwhile, send dial tone
		//(sequence of alternating 1's and 0's) at our speed (SATA I).
		s_wait_align: //5
			begin
				txdataout_int_next = prim_dial;
				txcharisk_int_next = prim_dial_k;
				if (align_detected && ~rxsigdet_r) begin
					state_next = s_send_align;
				end
				else if (state_time_count == 24'h80E8) //880us passed with no response
					state_next = s_send_comreset;
			end
			
		//Align: Send align to acknowledge this speed. Wait for disk to respond with a SYNC primitive,
		//establishing the link.
		s_send_align: //6
			begin
				txdataout_int_next = prim_align;
				txcharisk_int_next = prim_align_k;
            //Wait for SYNC
            //Make exception for WD drive, which doesn't seem to send sync
				if ((sync_detected || (rxcharisk == 4'b0001 && ~align_detected)) && rx_locked_r) begin
					state_next = s_link_ready;
				end
				if (state_time_count == 24'h80E8) //880us passed with no response
					state_next = s_send_comreset;
				if (rxsigdet_r) //drive has given up
					state_next = s_send_comreset;
			end
		
		//Link ready: Handshake completed. Start sending and receiving data.
		s_link_ready: //7
			begin
					link_ready_next = 1'b1;
					txdataout_int_next = prim_sync;
					//Delay txcharisk by 1 cycle
					txcharisk_int_next = txcharisk_in;
					oob_sig_en_next = 1'b0;
				//end
				
			end
		
		default: state_next = s_send_comreset;
				
	endcase
	//Skip the OOB sequence for loopback
	if (force_ready)
		state_next = s_link_ready;
	
	//reset state time counter when change states
	if (state_next != state)
		state_time_count_rst = 1'b1;
	else
		state_time_count_rst = 1'b0;	
		
end

always @ (posedge txclk or posedge reset)
begin : Seq_FSM
	if (reset) begin
		state <= s_idle;
		state_time_count <= 24'h0;
		link_ready <= 1'b0;
		comreset_send_p <= 1'b0;
		comwake_send_p <= 1'b0;
		detect_ack_p <= 1'b0;
		oob_sig_en <= 1'b1;
		txcharisk_int <= 4'b0000;
		txdataout_int <= 32'h0;
	end
	else begin
		state <= state_next;
		link_ready <= link_ready_next;
		comreset_send_p <= comreset_send_p_next;
		comwake_send_p <= comwake_send_p_next;
		detect_ack_p <= detect_ack_p_next;
		oob_sig_en <= oob_sig_en_next;
		txcharisk_int <= txcharisk_int_next;
		txdataout_int <= txdataout_int_next;
		
		//state time counter
		if (~state_time_count_rst)
			state_time_count <= state_time_count + 1;
		else
			state_time_count <= 24'h0;
	end
end

//SATA Specification requires 2 ALIGNs to be inserted every 256 DWORDS
//The following code is from Ashwin Mendon's oob_control.v module
//ALIGN Primitives transmitted every 256 DWORDS for speed alignment

reg [8:0] align_count;
reg align_en, align_en_r;

always@(posedge txclk or posedge reset)
begin : align_cnt
	if (reset) begin
		align_count <= 9'b0;
		align_en_r  <= 1'b0; //align_en gives one cycle notice of align; use _r to delay one cycle
	end 
	else begin
		align_en_r <= align_en;
		if (align_count < 9'h0FF) //255   
		begin
			if (align_count == 9'h001) //de-assert after 2 ALIGN primitives
			begin
				align_en <= 1'b0;
			end  
			align_count <= align_count + 1;
		end
		else begin     
			align_count <= 9'b0;
			align_en <= 1'b1;
		end
	end
end

//Output MUX
always @ (*)
begin
	//During the handshake sequence, let the state machine control the output
	if (link_ready != 1'b1) begin
		txdataout = txdataout_int;
		txcharisk = txcharisk_int;
	end
	//After link is ready, use data from upper layers or the inserted ALIGNs
	else begin
		if (align_en_r == 1'b1) begin
			txdataout = prim_align;
			txcharisk = prim_align_k;
		end
		else begin
			txdataout = txdatain;
			txcharisk = txcharisk_int; //Delayed by 1 cycle
		end
	end
end

assign txenoob = oob_sig_en && oob_txenoob;
assign align_en_out = align_en;
assign stateOut = state;

endmodule
