`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name:    sata_core
// Module Name:    sata_core 
// Project Name: 
// Target Devices: Virtex4
// Tool versions: ISE 14.2
// Description: Top level module for the SATA core. Based on sata_core.vhd by
// Ashwin Mendon.
//
// Port Descriptions:
//    -- Physical --
//
// UPPER_MGTCLK_PAD_N_IN, 
// UPPER_MGTCLK_PAD_P_IN   : Reference clock for MGTs
// RX1N_IN, RX1P_IN        : MGT Receive pair
// TX1N_OUT, TX1P_OUT      : MGT Transmit pair
// sys_clk                 : Clock that is independent from the SATA clock.
//                           This is 100MHz on the ML405.
// reset                   : Resets the entire SATA core, including data FIFOs
//
//    -- Command, Control and Status --
//
// ready_for_cmd           : When asserted, SATA core is ready to execute new command.
//                           This signal goes low after new_cmd is asserted.                    
//                           It also serves as the command done signal.
// new_cmd                 : Asserted for one clock cycle to start a request        
// cmd_type                : "01" for READ request and "10" for WRITE request	          
// sector_count            : Number of sectors requested by user. 1 Sector = 512 Bytes
// sector_addr             : Starting address of request
// sata_timer              : number of cycles to complete last operation,
//                         : to check performance.
//      			
//   -- Data and User Clock --
//
// sata_din                :  Data from user to Write to Disk  
// sata_din_we             :  Write Enable to SATA Core when FULL is low 
// sata_core_full          :  SATA Core Full- de-assert WE 
// sata_dout               :  Data output from SATA Core
// sata_dout_re            :  Read Enable from SATA asserted when EMPTY is low 
// sata_core_empty         :  SATA Core Empty- de-assert RE
// SATA_USER_DATA_CLK_IN   :  SATA Core Write Clock  
// SATA_USER_DATA_CLK_OUT  :  SATA Core Read Clock
//
// -- Debugging --
//
// *_ila_control           : Connect to a Chipscope ICON for debugging.
// SERIAL_RX, SERIAL_TX    : Connect to UART rx/tx if using the SATA event logger.
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module sata_core #
(
   parameter DEBUG             = 0, //Use debugging features
   parameter USE_REPLAY_BUFFER = 1, //Automatically recover from errors
   parameter USE_EVENT_LOGGER  = 0  //Debug SATA events over the serial port
)(
   //SATA Pins
   input UPPER_MGTCLK_PAD_N_IN,
   input UPPER_MGTCLK_PAD_P_IN,
   input [1:0] RX1N_IN,
   input [1:0] RX1P_IN,
   output [1:0] TX1N_OUT,
   output [1:0] TX1P_OUT,
   //Clock and Resets
   input sys_clk, //100MHz
   input reset,

   output ready_for_cmd,
   input new_cmd,
   input [1:0] cmd_type,
   input [31:0] sector_count,
   input [31:0] sector_addr,

   input [31:0] sata_din,
   input sata_din_we,
   output sata_core_full,
   output [31:0] sata_dout, 
   input sata_dout_re,
   output sata_core_empty,
   input SATA_USER_DATA_CLK_IN,
   output SATA_USER_DATA_CLK_OUT,

   output LINKUP,

   //Chipscope
   inout [35:0] sata_phy_ila_control,
   inout [35:0] cmd_layer_ila_control,
   inout [35:0] sata_tx_frame_ila_control,
   inout [35:0] sata_rx_frame_ila_control,
   inout [35:0] sata_loopback_ila_control,
   //Debugging
   input SERIAL_RX,
   output SERIAL_TX,
   output [47:0] sata_timer
);

//wire declarations
//Clock
wire sata_usr_clk;

wire active_mgt_rxlock;

wire write_fifo_full;
wire align_en;
wire [31:0] tx_data;
wire tx_charisk;
wire [31:0] rx_data;
wire [3:0] rx_charisk;
wire [3:0] oob_state;
wire rxelecidle;
wire linkup;
wire read_fifo_empty;
//Between the User Fifo and the Command Layer
wire [31:0] user_fifo_dout;
wire user_din_re;
wire user_fifo_prog_full;
wire user_fifo_empty;
wire user_fifo_full;
wire user_fifo_almost_full;
//Between Command Layer and Link Layer
wire ll_ready_for_cmd_i;
wire ll_cmd_start;
wire [1:0] ll_cmd_type;
wire [31:0] ll_dout, ll_din;
wire ll_dout_we, ll_din_re;
wire tx_err, tx_ok;
wire [31:0] curr_sector_count;
//Error Detection
wire link_error, link_error_ack;
wire sata_low_reset;
reg sata_hard_reset, sata_hard_reset_next;
wire sync_escape;
wire sync_escape_flag;

//SATA Core Output Signals
assign sata_core_full = user_fifo_prog_full;
assign sata_core_empty = read_fifo_empty;
assign LINKUP  = linkup; 
assign SATA_USER_DATA_CLK_OUT = sata_usr_clk;

//User FIFO: Data to be written to the disk
user_fifo user_fifo_i (
	.rst (reset),
	.wr_clk (SATA_USER_DATA_CLK_IN),
	.din (sata_din),
	.wr_en (sata_din_we),
	.rd_clk (sata_usr_clk),
	.dout (user_fifo_dout),
	.rd_en (user_din_re),
	.full (user_fifo_full),
	.prog_full (user_fifo_prog_full),
	.empty (user_fifo_empty)
   //.almost_full(user_fifo_almost_full)
);

command_layer #
(
   .CHIPSCOPE(DEBUG),
   .USE_REPLAY_BUFFER(USE_REPLAY_BUFFER)
) command_layer_i
(
    .clk 							(sata_usr_clk),
    .sw_reset 						(reset),
    .cmd_layer_ila_control 	(cmd_layer_ila_control),
    .new_cmd 						(new_cmd),
    .cmd_done 						(ready_for_cmd),
    .cmd_type 						(cmd_type),
    .sector_count 				(sector_count),
    .sector_addr 					(sector_addr),
    .user_din 						(user_fifo_dout),
    .user_din_re_out 			(user_din_re),
    .user_dout 					(sata_dout),
    .user_dout_re 				(sata_dout_re),
    .user_fifo_empty 			(user_fifo_empty),
    .user_fifo_full 				(user_fifo_prog_full),
    .sector_timer_out 			(sata_timer),
    // Signals from/to Link Layer
    .write_fifo_full  			(write_fifo_full),
    .ll_ready_for_cmd 			(ll_ready_for_cmd_i), 
    .ll_cmd_start 				(ll_cmd_start),
    .ll_cmd_type 					(ll_cmd_type),
    .ll_dout 						(ll_dout),
    .ll_dout_we 					(ll_dout_we),
    .ll_din 						(ll_din),
    .ll_din_re 					(ll_din_re),
	 .r_err_in						((rx_data == 32'h5656B57C)),
    .tx_err_in                (tx_err),
    .tx_ok_in                 (tx_ok),
    .sector_count_out         (curr_sector_count),
    .link_error               (link_error),
    .link_error_ack           (link_error_ack),
    .sata_low_reset           (sata_low_reset)
   );

sata_link_layer #
(
   .CHIPSCOPE(DEBUG),
   .USE_REPLAY_BUFFER (USE_REPLAY_BUFFER),
   .USE_EVENT_LOGGER (USE_EVENT_LOGGER)
) sata_link_layer_i
(
    .sw_reset									(reset | sata_low_reset),
    .sata_rx_frame_ila_control			(sata_rx_frame_ila_control),
    .sata_tx_frame_ila_control			(sata_tx_frame_ila_control),
    .scrambler_ila_control					(),
    .descrambler_ila_control				(),
    .ready_for_cmd_out						(ll_ready_for_cmd_i),
    .new_cmd_in								(ll_cmd_start),
    .cmd_type									(ll_cmd_type),
    .sector_count_in							(curr_sector_count),
    .sata_din									(ll_dout),
    .sata_din_we								(ll_dout_we),
    .sata_dout									(ll_din),
    .sata_dout_re								(ll_din_re),
    .read_fifo_empty							(read_fifo_empty),
    .write_fifo_full							(write_fifo_full),
	 .sata_user_clk							(sata_usr_clk),
 	 .align_en								   (align_en),	
    .tx_dataout								(tx_data),
    .tx_charisk_out							(tx_charisk),
	 .rx_datain									(rx_data),
	 .rx_charisk_in							(rx_charisk),
    .CurrentOobState							({4'b0, oob_state}),
    .rxelecidle								(rxelecidle),
	 .LINKUP										(linkup),	 
	 .sys_clk								   (sys_clk),
	 .SERIAL_RX									(SERIAL_RX),
	 .SERIAL_TX									(SERIAL_TX),
	 .rxlock										(active_mgt_rxlock),
    .tx_err_out                        (tx_err),
    .tx_ok_out                         (tx_ok)
      );
	 
SATA_PHY #
(
   .DEBUG (DEBUG),
   .LOOPBACK(0)
)
sata_phy_i 
(
	 .sata_phy_ila_control 			(sata_phy_ila_control),
	 .sata_loopback_ila_control   (sata_loopback_ila_control),
    .UPPER_MGTCLK_PAD_N_IN 		(UPPER_MGTCLK_PAD_N_IN),
    .UPPER_MGTCLK_PAD_P_IN 		(UPPER_MGTCLK_PAD_P_IN),
    .sys_clk 						   (sys_clk),
    .tx_system_reset_in				(reset | sata_low_reset),
    .rx_system_reset_in 			(reset | sata_low_reset),
    .active_mgt_rxlock           (active_mgt_rxlock),
    .RX1N_IN 							(RX1N_IN),
    .RX1P_IN 							(RX1P_IN),
    .TX1N_OUT 							(TX1N_OUT),
    .TX1P_OUT 							(TX1P_OUT),
	 
	 //Link Layer
   .tx_datain 							(tx_data),             
   .tx_charisk_in 					(tx_charisk),        
	.linkup 								(linkup),
	.align_en_out 						(align_en),
	.sata_user_clk 					(sata_usr_clk),
   .rx_dataout 						(rx_data),     
	.rx_charisk_out 					(rx_charisk),
	.rxelecidle_out 					(rxelecidle),
	.oob_state							(oob_state)
);

//--------------------------------
// Link Error Monitor
//--------------------------------
//Watch for error conditions. When they occur,
//issue a hard reset to the link layer and the physical layer,
//and notify the command layer. The command layer will then
//use its replay buffer to continue sending the data.
parameter [1:0] em_monitor = 2'b00,
                em_notify  = 2'b10;
                
reg [1:0] em_state, em_state_next;
reg [31:0] tx_data_d1, tx_data_d2;
reg [31:0] rx_data_d1, rx_data_d2;
reg rxsigdet_r;
always @ (posedge sata_usr_clk) begin
   tx_data_d1 <= tx_data;
   tx_data_d2 <= tx_data_d1;
   rx_data_d1 <= rx_data;
   rx_data_d2 <= rx_data_d1;
   rxsigdet_r <= rxelecidle;
end

always @ (*) begin
   em_state_next = em_state;
   sata_hard_reset_next = 1'b0;
   case (em_state)
      em_monitor: begin
         //Frame sequence break
         if ((rx_data == 32'h5858B57C) && (rx_data_d1 == 32'h5858B57C) 
             && (rx_charisk == 4'b0001)) begin
            if (((tx_data == 32'h4A4A957C) || (tx_data == 32'h5555B57C)) && (tx_charisk == 1'b1)) begin
               em_state_next = em_notify;
            end
         end
         //Loss of link
         if (linkup && rxsigdet_r) begin
            em_state_next = em_notify;
            sata_hard_reset_next = 1'b1;
         end
      end
      em_notify: begin
         //Wait for command layer to acknowledge the reset
         if (link_error_ack == 1'b1) begin
            em_state_next = em_monitor;
         end
      end
   endcase
end

always @ (posedge sata_usr_clk or posedge reset) begin
   if (reset) begin
      em_state <= em_monitor;
      sata_hard_reset <= 1'b0;
   end
   else begin
      em_state <= em_state_next;
      sata_hard_reset <= sata_hard_reset_next;
   end
end

assign link_error = (em_state == em_notify);

endmodule
