`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
//  
// Design Name:    sata_core
// Module Name:    sata_phy 
// Project Name: 
// Target Devices: Virtex-4
// Tool versions: ISE 14.2
// Description: Implements the physical layer of SATA. Instantiates a RocketIO
// MGT and has logic for handling link initialization. 
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module SATA_PHY #
(
   parameter DEBUG      =  1, //Use Chipscope
   parameter LOOPBACK   =  0  //Loopback testing
)(
   //Physical Pins
   input					UPPER_MGTCLK_PAD_N_IN,
   input					UPPER_MGTCLK_PAD_P_IN,
   input  [1:0]	   RX1N_IN,
   input  [1:0] 		RX1P_IN,
   output [1:0] 		TX1N_OUT,
   output [1:0] 		TX1P_OUT,
   //Clock and Resets
   input             sys_clk, //100MHz oscillator on ML405   
   input 				tx_system_reset_in,
   input  				rx_system_reset_in,
   //Status
   output 				active_mgt_rxlock,
   //Link Layer Interface
   input  [31:0]     tx_datain,             
   input             tx_charisk_in,        
   output [31:0]     rx_dataout,     
   output [3:0]      rx_charisk_out,
   output		      linkup,
   output		      align_en_out,
   output		      sata_user_clk,
   //Debugging signals
   output [3:0]	   oob_state,
   output			   rxelecidle_out,
   inout  [35:0]   	sata_phy_ila_control,
   inout  [35:0]		sata_loopback_ila_control
);

//Signal Declarations
wire                 refclk1_i;
wire                 drp_clk_i;
wire                 txlock_i;
wire                 tx_pmareset_c;
wire                 txreset_c;
wire                 tx_sync_i;
wire                 tx_system_ready_i, rx_system_ready_i;
wire                 rx_pmareset_c;
wire                 rxreset_c;
wire                 rx_sync_i;
wire [31:0]          mgt0_rxdata_i, mgt1_rxdata_i;
wire [31:0]          mgt0_txdata_i, mgt1_txdata_i;
wire [ 3:0]          mgt0_rxcharisk_i, mgt1_rxcharisk_i;
wire [ 3:0]          mgt0_txcharisk_i, mgt1_txcharisk_i;
wire                 mgt0_usr_clk2_i, mgt1_usr_clk2_i;
wire                 mgt0_rxusr_clk2_i, mgt1_rxusr_clk2_i;
wire                 mgt0_rxrecclk1_i, mgt1_rxrecclk1_i;
wire                 mgt0_txoutclk1_i, mgt1_txoutclk1_i;
wire                 mgt0_rxsigdet_i, mgt1_rxsigdet_i;
wire                 mgt0_txenoob_i, mgt1_txenoob_i;
wire                 mgt0_txrundisp_i, mgt1_txrundisp_i;
wire                 mgt0_rxrundisp_i, mgt1_rxrundisp_i;
wire                 mgt0_rxrealign_i, mgt1_rxrealign_i;
wire                 MGT0_RXLOCK_OUT, MGT0_TXLOCK_OUT;
wire                 MGT1_RXLOCK_OUT, MGT1_TXLOCK_OUT;

wire                 link_ready;

wire                 active_mgt_rxlock_i;
reg  [31:0]          active_mgt_rxdata_i;
wire [31:0]          active_mgt_rxdata_prebuff_i;
reg  [ 3:0]          active_mgt_rxcharisk_i;
wire [ 3:0]          active_mgt_rxcharisk_prebuff_i;
wire                 active_mgt_usr_clk, inactive_mgt_usr_clk;
wire				      active_mgt_rxrecclk1_i, inactive_mgt_rxrecclk1_i;
wire				      active_mgt_rxusr_clk;
wire				      active_mgt_txoutclk1_i;
wire                 active_mgt_rxsigdet_i;
wire                 active_mgt_txenoob_i;
wire [31:0]          active_mgt_txdata_i, inactive_mgt_txdata_i;
wire [ 3:0]          active_mgt_txcharisk_i, inactive_mgt_txcharisk_i;

//////////////////////////////////////////////////////////////////////////////////
// Active/Inactive
// Controls whether MGT0 or MGT1 is the active MGT. The active MGT is connected to
// the rest of the SATA core logic. The inactive MGT is connected to a small debug
// state machine for loopback debugging, if this option is enabled.
// To switch the active MGT, change all instances of "mgt0" to "mgt1" and vice
// versa.
//////////////////////////////////////////////////////////////////////////////////
assign               active_mgt_rxlock_i =            MGT0_RXLOCK_OUT;
assign               active_mgt_rxdata_prebuff_i =    mgt0_rxdata_i;
assign               active_mgt_rxcharisk_prebuff_i = mgt0_rxcharisk_i;
assign               active_mgt_rxrecclk1_i =         mgt0_rxrecclk1_i;
assign               active_mgt_txoutclk1_i =         mgt0_txoutclk1_i;
assign               active_mgt_rxsigdet_i =          mgt0_rxsigdet_i;
assign				   mgt0_usr_clk2_i =                active_mgt_usr_clk;
assign               mgt0_rxusr_clk2_i =              active_mgt_rxusr_clk;
assign               mgt0_txenoob_i =                 active_mgt_txenoob_i;
assign               mgt0_txdata_i =                  active_mgt_txdata_i;
assign               mgt0_txcharisk_i =               active_mgt_txcharisk_i;

assign               inactive_mgt_rxrecclk1_i =       mgt1_rxrecclk1_i;
assign               mgt1_usr_clk2_i =                inactive_mgt_usr_clk;
assign               mgt1_rxusr_clk2_i =              inactive_mgt_usr_clk;
assign               mgt1_txdata_i =                  inactive_mgt_txdata_i;
assign               mgt1_txcharisk_i =               inactive_mgt_txcharisk_i;
// -------- Static Assignments --------
assign               txlock_i = MGT0_TXLOCK_OUT & MGT1_TXLOCK_OUT;

//////////////////////////////////////////////////////////////////////////////////
// Create the DRP Clock by dividing the system clock by 2
// DRP clock must be <= 50MHz
// Be sure to set appropriately in the RocketIO Wizard based on your system clock
// The ML405 has a 100MHz system clock, so the DRP clock is 50Mhz
//////////////////////////////////////////////////////////////////////////////////

reg [1:0] drp_counter;
wire drp_clk_prebuff_i;

always@ (posedge sys_clk) drp_counter <= drp_counter + 1;
assign drp_clk_prebuff_i = drp_counter[1];
BUFG drp_clk_bufg_i ( .I (drp_clk_prebuff_i), .O (drp_clk_i));

//////////////////////////////////////////////////////////////////////////////////
// RocketIO Instantiation
// Created from the RocketIO Wizard template
//////////////////////////////////////////////////////////////////////////////////

defparam rocketio_i.SIMULATION_P = 0;
defparam rocketio_i.MGT0_GT11_MODE_P  = "B";
defparam rocketio_i.MGT0_MGT_ID_P     =   1;
defparam rocketio_i.MGT1_GT11_MODE_P  = "A";
defparam rocketio_i.MGT1_MGT_ID_P     =   0;
ROCKETIO rocketio_i
(

   //_____________________________________________________________________
   //_____________________________________________________________________
   //MGT0   (X1Y2)
   //--------------- 8B10B Receive Data Path and Control Ports ----------------
   .MGT0_RXCHARISCOMMA_OUT         ( ), //nc
   .MGT0_RXCHARISK_OUT             (mgt0_rxcharisk_i),
   .MGT0_RXDATA_OUT                (mgt0_rxdata_i),
   .MGT0_RXDISPERR_OUT             ( ), //nc
   .MGT0_RXNOTINTABLE_OUT          ( ), //nc
   .MGT0_RXRUNDISP_OUT             (mgt0_rxrundisp_i),
   //--------------- 8B10B Transmit Data Path and Control Ports ---------------
   .MGT0_TXBYPASS8B10B_IN          (4'b0),
   .MGT0_TXCHARDISPMODE_IN         (4'b0),
   .MGT0_TXCHARDISPVAL_IN          (4'b0),
   .MGT0_TXCHARISK_IN              (mgt0_txcharisk_i),
   .MGT0_TXDATA_IN                 (mgt0_txdata_i),
   .MGT0_TXKERR_OUT                ( ), //nc
   .MGT0_TXRUNDISP_OUT             (mgt0_txrundisp_i),
   //------------------------ Calibration Block Ports -------------------------
   .MGT0_ACTIVE_OUT                ( ), //nc
   .MGT0_DISABLE_IN                (1'b0),
   .MGT0_DRP_RESET_IN              (tx_system_reset_in | rx_system_reset_in),
   .MGT0_RX_SIGNAL_DETECT_IN       (1'b1),
   .MGT0_TX_SIGNAL_DETECT_IN       (1'b1),
   //--------------------------- Calibration Ports ----------------------------
   .MGT0_RXCLKSTABLE_IN            (1'b1),
   .MGT0_TXCLKSTABLE_IN            (1'b1),
   //------------------- Dynamic Reconfiguration Port (DRP) -------------------
   .MGT0_DADDR_IN                  (8'b0),
   .MGT0_DCLK_IN                   (drp_clk_i),
   .MGT0_DEN_IN                    (1'b0),
   .MGT0_DI_IN                     (16'b0),
   .MGT0_DO_OUT                    ( ), //nc
   .MGT0_DRDY_OUT                  ( ), //nc
   .MGT0_DWE_IN                    (1'b0), 
   //------------------------------ Global Ports ------------------------------
   .MGT0_LOOPBACK_IN               (2'b0),
   .MGT0_POWERDOWN_IN              (1'b0),
   .MGT0_TXINHIBIT_IN              (1'b0),
   //---------------------- Out of Band Signalling Ports ----------------------
   .MGT0_RXSIGDET_OUT              (mgt0_rxsigdet_i),
   .MGT0_TXENOOB_IN                (mgt0_txenoob_i),
   //-------------------------------- PLL Lock --------------------------------
   .MGT0_RXLOCK_OUT                (MGT0_RXLOCK_OUT),
   .MGT0_TXLOCK_OUT                (MGT0_TXLOCK_OUT),
   //------------------------- Polarity Control Ports -------------------------
   .MGT0_RXPOLARITY_IN             (1'b0),
   .MGT0_TXPOLARITY_IN             (1'b0),
   //-------------------------- Ports for Simulation --------------------------
   .MGT0_COMBUSIN_IN               ( ),
   .MGT0_COMBUSOUT_OUT             ( ),
   //---------------------------- Reference Clocks ----------------------------
   .MGT0_REFCLK1_IN                (refclk1_i),
   //--------------------------------- Resets ---------------------------------
   .MGT0_RXPMARESET_IN             (rx_pmareset_c),
   .MGT0_RXRESET_IN                (rxreset_c),
   .MGT0_TXPMARESET_IN             (tx_pmareset_c),
   .MGT0_TXRESET_IN                (txreset_c),
   //---------------------------- Serdes Alignment ----------------------------
   .MGT0_ENMCOMMAALIGN_IN          (1'b1),
   .MGT0_ENPCOMMAALIGN_IN          (1'b1),
   .MGT0_RXCOMMADET_OUT            ( ),
   .MGT0_RXREALIGN_OUT             (mgt0_rxrealign_i),
   //------------------------------ Serial Ports ------------------------------
   .MGT0_RX1N_IN                   (RX1N_IN[0]),
   .MGT0_RX1P_IN                   (RX1P_IN[0]),
   .MGT0_TX1N_OUT                  (TX1N_OUT[0]),
   .MGT0_TX1P_OUT                  (TX1P_OUT[0]),
   //--------------------------------- Status ---------------------------------
   .MGT0_RXSTATUS_OUT              ( ),
   //---------------------------- Synchronization -----------------------------
   .MGT0_RXSYNC_IN                 (rx_sync_i),
   .MGT0_TXSYNC_IN                 (tx_sync_i),
   //------------------------------ User Clocks -------------------------------
   .MGT0_RXRECCLK1_OUT             (mgt0_rxrecclk1_i),
   .MGT0_RXRECCLK2_OUT             ( ), //Do not use, as per User Guide p. 70
   .MGT0_RXUSRCLK2_IN              (mgt0_rxusr_clk2_i),
   .MGT0_TXOUTCLK1_OUT             (mgt0_txoutclk1_i),
   .MGT0_TXOUTCLK2_OUT             ( ), //Do not use, as per User Guide p. 68
   .MGT0_TXUSRCLK2_IN              (mgt0_usr_clk2_i),


   //_____________________________________________________________________
   //_____________________________________________________________________
   //MGT1   (X1Y3)
   //--------------- 8B10B Receive Data Path and Control Ports ----------------
   .MGT1_RXCHARISCOMMA_OUT         ( ),
   .MGT1_RXCHARISK_OUT             (mgt1_rxcharisk_i),
   .MGT1_RXDATA_OUT                (mgt1_rxdata_i),
   .MGT1_RXDISPERR_OUT             ( ),
   .MGT1_RXNOTINTABLE_OUT          ( ),
   .MGT1_RXRUNDISP_OUT             (mgt1_rxrundisp_i),
   //--------------- 8B10B Transmit Data Path and Control Ports ---------------
   .MGT1_TXBYPASS8B10B_IN          (4'b0),
   .MGT1_TXCHARDISPMODE_IN         (4'b0),
   .MGT1_TXCHARDISPVAL_IN          (4'b0),
   .MGT1_TXCHARISK_IN              (mgt1_txcharisk_i),
   .MGT1_TXDATA_IN                 (mgt1_txdata_i),
   .MGT1_TXKERR_OUT                ( ),
   .MGT1_TXRUNDISP_OUT             (mgt1_txrundisp_i),
   //------------------------ Calibration Block Ports -------------------------
   .MGT1_ACTIVE_OUT                ( ),
   .MGT1_DISABLE_IN                (1'b0),
   .MGT1_DRP_RESET_IN              (tx_system_reset_in | rx_system_reset_in),
   .MGT1_RX_SIGNAL_DETECT_IN       (1'b1),
   .MGT1_TX_SIGNAL_DETECT_IN       (1'b1),
   //--------------------------- Calibration Ports ----------------------------
   .MGT1_RXCLKSTABLE_IN            (1'b1),
   .MGT1_TXCLKSTABLE_IN            (1'b1),
   //------------------- Dynamic Reconfiguration Port (DRP) -------------------
   .MGT1_DADDR_IN                  (8'b0),
   .MGT1_DCLK_IN                   (drp_clk_i),
   .MGT1_DEN_IN                    (1'b0),
   .MGT1_DI_IN                     (16'b0),
   .MGT1_DO_OUT                    ( ),
   .MGT1_DRDY_OUT                  ( ),
   .MGT1_DWE_IN                    (1'b0),
   //------------------------------ Global Ports ------------------------------
   .MGT1_LOOPBACK_IN               (2'b0),
   .MGT1_POWERDOWN_IN              (1'b0),
   .MGT1_TXINHIBIT_IN              (1'b0),
   //---------------------- Out of Band Signalling Ports ----------------------
   .MGT1_RXSIGDET_OUT              (mgt1_rxsigdet_i),
   .MGT1_TXENOOB_IN                (mgt1_txenoob_i),
   //-------------------------------- PLL Lock --------------------------------
   .MGT1_RXLOCK_OUT                (MGT1_RXLOCK_OUT),
   .MGT1_TXLOCK_OUT                (MGT1_TXLOCK_OUT),
   //------------------------- Polarity Control Ports -------------------------
   .MGT1_RXPOLARITY_IN             (1'b0),
   .MGT1_TXPOLARITY_IN             (1'b0),
   //-------------------------- Ports for Simulation --------------------------
   .MGT1_COMBUSIN_IN               ( ),
   .MGT1_COMBUSOUT_OUT             ( ),
   //---------------------------- Reference Clocks ----------------------------
   .MGT1_REFCLK1_IN                (refclk1_i),
   //--------------------------------- Resets ---------------------------------
   .MGT1_RXPMARESET_IN             (rx_pmareset_c),
   .MGT1_RXRESET_IN                (rxreset_c),
   .MGT1_TXPMARESET_IN             (tx_pmareset_c),
   .MGT1_TXRESET_IN                (txreset_c),
   //---------------------------- Serdes Alignment ----------------------------
   .MGT1_ENMCOMMAALIGN_IN          (1'b1),
   .MGT1_ENPCOMMAALIGN_IN          (1'b1),
   .MGT1_RXCOMMADET_OUT            ( ),
   .MGT1_RXREALIGN_OUT             (mgt1_rxrealign_i),
   //------------------------------ Serial Ports ------------------------------
   .MGT1_RX1N_IN                   (RX1N_IN[1]),
   .MGT1_RX1P_IN                   (RX1P_IN[1]),
   .MGT1_TX1N_OUT                  (TX1N_OUT[1]),
   .MGT1_TX1P_OUT                  (TX1P_OUT[1]),
   //--------------------------------- Status ---------------------------------
   .MGT1_RXSTATUS_OUT              ( ),
   //---------------------------- Synchronization -----------------------------
   .MGT1_RXSYNC_IN                 (rx_sync_i),
   .MGT1_TXSYNC_IN                 (tx_sync_i),
   //------------------------------ User Clocks -------------------------------
   .MGT1_RXRECCLK1_OUT             (mgt1_rxrecclk1_i),
   .MGT1_RXRECCLK2_OUT             ( ), //Do not use, as per User Guide p. 70
   .MGT1_RXUSRCLK2_IN              (mgt1_rxusr_clk2_i),
   .MGT1_TXOUTCLK1_OUT             (mgt1_txoutclk1_i),
   .MGT1_TXOUTCLK2_OUT             ( ), //Do not use, as per User Guide p. 68
   .MGT1_TXUSRCLK2_IN              (mgt1_usr_clk2_i)

);

//********************** MGT clock module for refclk routing ******************
MGT_CLOCK_MODULE mgt_clock_module_i
(
   //----------------- Input Differential Clocks from Pads ---------------
   .UPPER_MGTCLK_PAD_N_IN          (UPPER_MGTCLK_PAD_N_IN),
   .UPPER_MGTCLK_PAD_P_IN          (UPPER_MGTCLK_PAD_P_IN),
   //----------------------- Output Reference Clocks ---------------------
   .REFCLK1_OUT                    (refclk1_i)
);

//--------------------- Instantiate an GT11_INIT module -------------------
// This module is responsible for performing the correct reset and lock procedure 
// as recommended in the user guide

GT11_INIT_TX #
(
   .C_SIMULATION (0)
)
gt11_init_tx_i 
(
   .CLK          (drp_clk_i),   
   .START_INIT   (tx_system_reset_in),      
   .USRCLK_STABLE(1'b1),
   .LOCK         (txlock_i),    
   .PCS_ERROR    (1'b0), //No MGT buffer used in this design
   .PMA_RESET    (tx_pmareset_c),  
   .PCS_RESET    (txreset_c),          
   .SYNC         (tx_sync_i),
   .READY        (tx_system_ready_i)   
);


GT11_INIT_RX #
(
   .C_SIMULATION (0)
)
gt11_init_rx_i 
(
   .CLK          (drp_clk_i),  
   .START_INIT   (rx_system_reset_in),  
   .USRCLK_STABLE(1'b1),
   .LOCK         (active_mgt_rxlock_i),   
   .PCS_ERROR    (1'b0), //No MGT buffer used in this design  
   .PMA_RESET    (rx_pmareset_c), 
   .PCS_RESET    (rxreset_c),         
   .SYNC         (rx_sync_i),
   .READY        (rx_system_ready_i)
);

//////////////////////////////////////////////////////////////////////////////////
// Buffer for the received data -- Synchronizes the data with the TX clock.
// The RX clock is derived from the incoming bitstream and runs slightly slower
// than the TX clock. If the TX clock is used to drive the system, some rxdata
// will appear mangled if this buffer was not used. The RX clock is too unstable
// to drive the system, resulting in link errors.
//////////////////////////////////////////////////////////////////////////////////

wire [35:0] rxbuffer_out;
wire rxbuffer_valid;

rxdata_buffer rxdata_buffer_i (
   .rst (rx_system_reset_in_i),
   .wr_clk (active_mgt_rxusr_clk),
   .rd_clk (active_mgt_usr_clk),
   .din ({active_mgt_rxdata_prebuff_i, active_mgt_rxcharisk_prebuff_i}),
   .wr_en (1'b1),
   .rd_en (1'b1),
   .dout (rxbuffer_out),
   .full (), //nc
   .empty (), //nc
   .valid (rxbuffer_valid)
);
// The RX clock runs slightly slower than the TX clock because of spread spectrum
// clocking. When the buffer underflows, insert an ALIGN primitive. This will be
// ignored by the link layer, so this is safe.
always @ (posedge active_mgt_usr_clk) begin
   if (rxbuffer_valid) begin
      active_mgt_rxdata_i <= rxbuffer_out[35:4];
      active_mgt_rxcharisk_i <= rxbuffer_out[3:0];
   end
   else begin
      active_mgt_rxdata_i <= 32'h7B4A4ABC; //ALIGN primitive
      active_mgt_rxcharisk_i <= 4'b0001;
   end
end

//////////////////////////////////////////////////////////////////////////////////
// MGT Clocking
//-Each MGT has internal clock dividers and yields a TX clock from the oscillator
// and an RX clock that is derived from the incoming bitstream. These clocks are 
// buffered here and used to clock the interface between the FPGA fabric and the 
// MGTs. See page 61 of the RocketIO User Guide for more information.
//-One clock is used for the inactive MGT's interface, while separate RX and TX
// clocks are maintained for the active MGT.
//-The active MGT TX clock (active_mgt_usr_clk) drives the entire SATA core.
//-For SATA I speeds, these clocks are 37.5MHz
//////////////////////////////////////////////////////////////////////////////////
BUFG inactive_usr_clk_bufg_i (.I(inactive_mgt_rxrecclk1_i), .O(inactive_mgt_usr_clk));
BUFG active_rxusr_clk_bufg_i ( .I(active_mgt_rxrecclk1_i), .O(active_mgt_rxusr_clk));
//Clock for the SATA core
BUFG active_usr_clk_bufg_i ( .I(active_mgt_txoutclk1_i), .O(active_mgt_usr_clk));

//////////////////////////////////////////////////////////////////////////////////
// OOB Controller
// Handles the link initialization sequence and inserts ALIGNs every 256 DWords
//////////////////////////////////////////////////////////////////////////////////
oob_sequence_controller oob_seq_i
//# ( SYS_CLOCK_SPEED = 100) //TODO
(
   .sys_clk       (sys_clk),
   .txclk         (active_mgt_usr_clk),
   .rxclk         (active_mgt_usr_clk),
   .reset         (tx_system_reset_in),
   .link_reset    (tx_system_ready_i), //Begin link init when MGT is ready
   .rxsigdet      (active_mgt_rxsigdet_i),
   .rx_locked     (active_mgt_rxlock_i),
   .txenoob       (active_mgt_txenoob_i),
   .link_ready    (link_ready),
   .rxdatain      (active_mgt_rxdata_i),
   .txdataout     (active_mgt_txdata_i),
   .rxcharisk     (active_mgt_rxcharisk_i),
   .txcharisk     (active_mgt_txcharisk_i),
   .txdatain      (tx_datain), //Data from link layer to be sent
   .txcharisk_in  ({3'b0, tx_charisk_in}),
   .align_en_out  (align_en_out),
   .stateOut      (oob_state),
   .force_ready   (LOOPBACK)
);

//////////////////////////////////////////////////////////////////////////////////
// Output Assignments
//////////////////////////////////////////////////////////////////////////////////

assign rx_dataout = 			   active_mgt_rxdata_i;
assign rx_charisk_out = 		active_mgt_rxcharisk_i;
assign rxelecidle_out = 		active_mgt_rxsigdet_i;
assign linkup =					link_ready;
assign sata_user_clk = 		   active_mgt_usr_clk;
assign active_mgt_rxlock =    active_mgt_rxlock_i;

//////////////////////////////////////////////////////////////////////////////////
// Debugging: Chipscope and loopback testing
// Use the SATA crossover cable to perform loopback testing
//////////////////////////////////////////////////////////////////////////////////
generate
if (DEBUG==1) 
begin : debug
// -------- Chipscope ILA --------
//sata_phy_ila phy_ila_i (
//   .CONTROL (sata_phy_ila_control),
//   .CLK     (active_mgt_usr_clk),
//   .TRIG0   (active_mgt_txenoob_i), //1-bit wide triggers
//   .TRIG1   (active_mgt_rxsigdet_i),
//   .TRIG2   (link_ready),
//   .TRIG3   (active_mgt_rxlock_i),
//   .TRIG4   (active_mgt_rxcharisk_i), //4 bits wide
//   .TRIG5   ({tx_system_reset_in, oob_state[2:0]}),
//   .TRIG6   (active_mgt_rxdata_i), //32 bits wide
//   .TRIG7   (active_mgt_txdata_i),
//   .TRIG8   (tx_system_ready_i),
//   .TRIG9   (txlock_i),
//   .TRIG10  (rxbuffer_valid)
//);
end
endgenerate

generate
if (LOOPBACK==1) 
begin : loopback
loopback_phy_ila ila_i (
  .CONTROL (sata_loopback_ila_control),
  .CLK     (active_mgt_usr_clk),
  .TRIG0   (mgt0_txdata_i),
  .TRIG1   (mgt1_txdata_i),  
  .TRIG2   (mgt0_rxdata_i),
  .TRIG3   (mgt1_rxdata_i),
  .TRIG4   (mgt0_txcharisk_i),
  .TRIG5   (mgt1_txcharisk_i),
  .TRIG6   (mgt0_rxcharisk_i),
  .TRIG7   (mgt1_rxcharisk_i),
  .TRIG8   (mgt0_txrundisp_i),
  .TRIG9   (mgt1_rxrundisp_i),
  .TRIG10  (mgt0_rxrealign_i)
);

//Simple loopback state machine.
loopback_test loopback_test_i (
  .usr_clk    (inactive_mgt_usr_clk),
  .reset      (tx_system_reset_in),
  .txdata     (inactive_mgt_txdata_i),
  .txcharisk  (inactive_mgt_txcharisk_i)
);
end
endgenerate

//sata_phy_ila phy_ila_i (
//   .CONTROL (sata_phy_ila_control),
//   .CLK     (active_mgt_usr_clk),
//   .TRIG0   (active_mgt_txenoob_i), //1-bit wide triggers
//   .TRIG1   (active_mgt_rxsigdet_i),
//   .TRIG2   (link_ready),
//   .TRIG3   (active_mgt_rxlock_i),
//   .TRIG4   (active_mgt_rxcharisk_i), //4 bits wide
//   .TRIG5   ({tx_system_reset_in, oob_state[2:0]}),
//   .TRIG6   (active_mgt_rxdata_i), //32 bits wide
//   .TRIG7   (active_mgt_txdata_i),
//   .TRIG8   (tx_system_ready_i),
//   .TRIG9   (txlock_i),
//   .TRIG10  (rxbuffer_valid)
//);

endmodule
