`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name:    sata_core
// Module Name:    sata_event_logger 
// Project Name: 
// Target Devices: Virtex 4
// Tool versions: ISE 14.2
// Description: Monitors the link layer for events. Sends a short message over
// serial port when events occur. Can monitor the rx datapath, tx datapath,
// master state machine, and primitives being received.
// Since SATA runs much faster than the UART, overflows often occur. The first bit
// of the timestamp indicates an overflow. It is recommended to only monitor 1 or 2
// things at once and comment out the other monitors.
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module sata_event_logger(
   input sys_clk,
   input sata_clk,
   input RESET,
   input RX,
   output TX,
   input [3:0] master_fsm_value,
   input [3:0] tx_value,
   input [3:0] rx_value,
   input [31:0] rx_data,
   input [3:0] rxcharisk,
   input linkup,
   input rxlock
);

//Convert one Hex digit to its ASCII representation
function [7:0] nibbleToASCII;
input [3:0] nibble;
begin
	if (nibble == 4'h0) nibbleToASCII = 8'h30;
	else if (nibble == 4'h1) nibbleToASCII = 8'h31;
	else if (nibble == 4'h2) nibbleToASCII = 8'h32;
	else if (nibble == 4'h3) nibbleToASCII = 8'h33;
	else if (nibble == 4'h4) nibbleToASCII = 8'h34;
	else if (nibble == 4'h5) nibbleToASCII = 8'h35;
	else if (nibble == 4'h6) nibbleToASCII = 8'h36;
	else if (nibble == 4'h7) nibbleToASCII = 8'h37;
	else if (nibble == 4'h8) nibbleToASCII = 8'h38;
	else if (nibble == 4'h9) nibbleToASCII = 8'h39;
	else if (nibble == 4'hA) nibbleToASCII = 8'h41;
	else if (nibble == 4'hB) nibbleToASCII = 8'h42;
	else if (nibble == 4'hC) nibbleToASCII = 8'h43;
	else if (nibble == 4'hD) nibbleToASCII = 8'h44;
	else if (nibble == 4'hE) nibbleToASCII = 8'h45;
	else nibbleToASCII = 8'h46;
end
endfunction
	 
reg rx_en, tx_en, master_en, prim_en;
reg [63:0] rx_message, tx_message, master_message, prim_message;

serial_top serial_top_i (
	.sys_clk (sys_clk),
	.sata_clk (sata_clk),
	.reset (RESET),
	.serial_rx (RX),
	.serial_tx (TX),
	.rx_message (rx_message),
	.rx_en (rx_en),
	.tx_message (tx_message),
	.tx_en (tx_en),
	.master_message (master_message),
	.master_en (master_en),
	.prim_message (prim_message),
	.prim_en (prim_en)
);

//Primitive monitor
reg [31:0] last_prim, last_prim_next;
reg [63:0] prim_message_next;
//reg link_lost, link_lost_next;
reg prim_en_next;
reg [1:0] prim_state, prim_state_next;
parameter ps_idle = 2'b00,
			 ps_monitor = 2'b01,
			 ps_linkdown = 2'b10;

always @ (*)
begin
	last_prim_next = last_prim;
	prim_message_next = prim_message;
	prim_state_next = prim_state;
	prim_en_next = 1'b0;
	case (prim_state)
		ps_idle: begin
			if (linkup)
				prim_state_next = ps_monitor;
		end
		ps_monitor: begin
			if ((rxcharisk != 4'b0000) && (rxcharisk != 4'b0001)) begin
				prim_state_next = ps_linkdown;
				if (!rxlock)
					prim_message_next = "LOCKLOST";
				else
					prim_message_next = "LINKLOST";
				prim_en_next = 1'b1;
			end
			else if ((rx_data != last_prim) && (rxcharisk == 4'b0001)) begin
				//last_prim_next = rx_data;
				case (rx_data)
					32'h7B4A4ABC: begin
						prim_message_next = "PR_ALIGN";
						prim_en_next = 1'b0; //don't track aligns
					end
					32'hB5B5957C: begin
						prim_message_next = "PR__SYNC";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h4A4A957C: begin
						prim_message_next = "PR_R_RDY";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h5555B57C: begin
						prim_message_next = "PR__R_IP";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h3535B57C: begin
						prim_message_next = "PR__R_OK";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h5656B57C: begin
						prim_message_next = "PR_R_ERR";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h3737B57C: begin
						prim_message_next = "PR___SOF";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'hD5D5B57C: begin
						prim_message_next = "PR___EOF";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h5757B57C: begin
						prim_message_next = "PR_X_RDY";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h5858B57C: begin
						prim_message_next = "PR__WTRM";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h9999AA7C: begin
						prim_message_next = "PR__CONT";
						prim_en_next = 1'b0; //do not track CONT
					end
					32'hD5D5AA7C: begin
						prim_message_next = "PR__HOLD";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					32'h9595AA7C: begin
						prim_message_next = "PRHOLDAK";
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
					default: begin
						//prim_message_next = "PUNKNOWN";
						prim_message_next[63:56] = nibbleToASCII(rx_data[31:28]);
						prim_message_next[55:48] = nibbleToASCII(rx_data[27:24]);
						prim_message_next[47:40] = nibbleToASCII(rx_data[23:20]);
						prim_message_next[39:32] = nibbleToASCII(rx_data[19:16]);
						prim_message_next[31:24] = nibbleToASCII(rx_data[15:12]);
						prim_message_next[23:16] = nibbleToASCII(rx_data[11: 8]);
						prim_message_next[15: 8] = nibbleToASCII(rx_data[ 7: 4]);
						prim_message_next[ 7: 0] = nibbleToASCII(rx_data[ 3: 0]);
						prim_en_next = 1'b1;
						last_prim_next = rx_data;
					end
				endcase
			end
		end
		
		ps_linkdown: begin
			if ((rxcharisk == 4'b0000) || (rxcharisk == 4'b0001)) begin
				if (rxlock) begin
					//prim_message_next = "LINKBACK";
               prim_message_next[63:56] = nibbleToASCII(rx_data[31:28]);
               prim_message_next[55:48] = nibbleToASCII(rx_data[27:24]);
               prim_message_next[47:40] = nibbleToASCII(rx_data[23:20]);
               prim_message_next[39:32] = nibbleToASCII(rx_data[19:16]);
               prim_message_next[31:24] = nibbleToASCII(rx_data[15:12]);
               prim_message_next[23:16] = nibbleToASCII(rx_data[11: 8]);
               prim_message_next[15: 8] = nibbleToASCII(rx_data[ 7: 4]);
               prim_message_next[ 7: 0] = nibbleToASCII(rx_data[ 3: 0]);
					prim_en_next = 1'b1;
					prim_state_next = ps_monitor;
				end
			end
		end
	endcase
end

always @ (posedge sata_clk or posedge RESET)
begin
	if (RESET) begin
		last_prim <= 32'h0;
		prim_message <= "PR_RESET";
		prim_state <= ps_idle;
		prim_en <= 1'b0;
	end
	else begin
		last_prim <= last_prim_next;
		prim_message <= prim_message_next;
		prim_state <= prim_state_next;
		prim_en <= prim_en_next;
	end
end


//master FSM monitor
reg [3:0] last_master;

always @ (posedge sata_clk or posedge RESET)
begin
	if (RESET) begin
		last_master = 4'hF;
		master_message = 64'h0;
		master_en = 1'b0;
	end
	else begin
		if (master_fsm_value != last_master) begin
			last_master = master_fsm_value;
			case (master_fsm_value)
				4'h0: begin
					master_message = "MAS_IDLE";
					master_en = 1'b1;
				end
				4'h1: begin
					master_message = "M_DEVSGN";
					master_en = 1'b1;
				end
				4'h2: begin
					master_message = "MWAITCMD";
					master_en = 1'b1;
				end
				4'h3: begin
					master_message = "M_H2D_RG";
					master_en = 1'b1;
				end
				4'h4: begin
					master_message = "M_DMAACT";
					master_en = 1'b1;
				end
				4'h5: begin
					master_message = "MHOSDATA";
					master_en = 1'b1;
				end
				4'h6: begin
					master_message = "MDEVDATA";
					master_en = 1'b1;
				end
				4'h7: begin
					master_message = "M_D2H_RG";
					master_en = 1'b1;
				end
				4'h8: begin
					master_message = "M_PIO_ST";
					master_en = 1'b1;
				end
				4'h9: begin
					master_message = "M_DEAD__";
					master_en = 1'b1;
				end
				4'hA: begin
					master_message = "M_OTHERS";
					master_en = 1'b1;
				end
				default: begin
					master_message = "M_ERROR!";
					master_en = 1'b1;
				end
			endcase
		end
		else begin
			master_message = "MAS45678";
			master_en = 1'b0;
		end
	end
end

//Monitor the received FIS Types
reg [2:0] fis_state, fis_state_next;
reg [63:0] rx_message_next;
reg rx_en_next;
parameter fis_wait = 3'b0,
          fis_eval = 3'b1;	  

always @ (*) begin
	fis_state_next = fis_state;
   rx_en_next = 1'b0;
	case (fis_state)
		fis_wait: begin
         if (rx_data == 32'h3737B57C) begin
            fis_state_next = fis_eval;
         end
      end
      
      fis_eval: begin
         fis_state_next = fis_wait;
         case (rx_data[7:0])
            8'hAA: begin
               rx_message_next = "FIS_RH2D";
               rx_en_next = 1'b1;
            end
            8'hB9: begin 
               rx_message_next = "FIS_RD2H";
               rx_en_next = 1'b1;
            end
            8'hB4: begin 
               rx_message_next = "FIS_DMAA";
               rx_en_next = 1'b1;
            end
            8'hCC: begin
               rx_message_next = "FISDMASU";
               rx_en_next = 1'b1;
            end
            8'hCB: begin
               rx_message_next = "FIS_DATA";
               rx_en_next = 1'b1;
            end
            8'hD5: begin
               rx_message_next = "FIS_BIST";
               rx_en_next = 1'b1;
            end
            8'hD2: begin
               rx_message_next = "FIS_PIOS";
               rx_en_next = 1'b1;
            end
            8'h2C: begin
               rx_message_next = "FSSETDEV";
               rx_en_next = 1'b1;
            end
            default: begin
               rx_message_next = "FIS_UNKN";
               rx_en_next = 1'b1;
            end
         endcase
      end
   endcase
end

always @ (posedge sata_clk or posedge RESET) begin
   if (RESET) begin
      fis_state <= fis_wait;
      rx_message <= "RX__DEAD";
      rx_en <= 1'b0;
   end
   else begin
      fis_state <= fis_state_next;
      rx_message <= rx_message_next;
      rx_en <= rx_en_next;
   end
end

////TX Datapath monitor
//reg [3:0] last_txval;
//
//always @ (posedge sata_clk or posedge RESET)
//begin
//	if (RESET) begin
//		last_txval = 4'hF;
//		tx_message = 64'h0;
//		tx_en = 1'b0;
//	end
//	else begin
//		if (tx_value != last_txval) begin
//			last_txval = tx_value;
//			case (tx_value)
//				4'h0: begin
//					tx_message = "TX__IDLE";
//					tx_en = 1'b1;
//				end
//				4'h1: begin
//					tx_message = "TX_SXRDY";
//					tx_en = 1'b1;
//				end
//				4'h2: begin
//					tx_message = "TX_S_SOF";
//					tx_en = 1'b1;
//				end
//				4'h3: begin
//					tx_message = "TX_S_FIS";
//					tx_en = 1'b1;
//				end
//				4'h4: begin
//					tx_message = "TX_S_EOF";
//					tx_en = 1'b1;
//				end
//				4'h5: begin
//					tx_message = "TX_SWTRM";
//					tx_en = 1'b1;
//				end
//				4'h6: begin
//					tx_message = "TX_SSYNC";
//					tx_en = 1'b1;
//				end
//				4'h7: begin
//					tx_message = "TX_SHLDA";
//					tx_en = 1'b1;
//				end
//				4'h8: begin
//					tx_message = "TX_SHOLD";
//					tx_en = 1'b1;
//				end
//				4'h9: begin
//					tx_message = "TX__DEAD";
//					tx_en = 1'b1;
//				end
//				default: begin
//					tx_message = "T_ERROR!";
//					tx_en = 1'b1;
//				end
//			endcase
//		end
//		else begin
//			tx_message = "TX345678";
//			tx_en = 1'b0;
//		end
//	end
//end

////RX Datapath monitor
//reg [3:0] last_rxval;
//
//always @ (posedge sata_clk or posedge RESET)
//begin
//	if (RESET) begin
//		last_rxval = 4'hF;
//		rx_message = 64'h0;
//		rx_en = 1'b0;
//	end
//	else begin
//		if (rx_value != last_rxval) begin
//			last_rxval = rx_value;
//			case (rx_value)
//				4'h0: begin
//					rx_message = "RX__IDLE";
//					rx_en = 1'b1;
//				end
//				4'h1: begin
//					rx_message = "RX_SRRDY";
//					rx_en = 1'b1;
//				end
//				4'h2: begin
//					rx_message = "RX_SR_IP";
//					rx_en = 1'b1;
//				end
//				4'h3: begin
//					rx_message = "RXSHLDAK";
//					rx_en = 1'b1;
//				end
//				4'h4: begin
//					rx_message = "RX_SR_OK";
//					rx_en = 1'b1;
//				end
//				4'h5: begin
//					rx_message = "RX_SSYNC";
//					rx_en = 1'b1;
//				end
//				4'h6: begin
//					rx_message = "RXW8XRDY";
//					rx_en = 1'b1;
//				end
//				default: begin
//					rx_message = "R_ERROR!";
//					rx_en = 1'b1;
//				end
//			endcase
//		end
//		else begin
//			rx_message = "RX345678";
//			rx_en = 1'b0;
//		end
//	end
//end
//					


endmodule
