`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name: 
// Module Name:    serial_debugger 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments:

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.   
//
//////////////////////////////////////////////////////////////////////////////////
module serial_arbiter (
    input clk,
    input reset,
	 //Interface to the debugging modules
    input [63:0] rx_message,
    input rx_en,
    input [63:0] tx_message,
    input tx_en,
    input [63:0] master_message,
    input master_en,
    input [63:0] prim_message,
    input prim_en,
	 //interface to the Message FIFO
	 output reg [127:0] message,
	 output reg message_wr_en,
	 input message_fifo_full
    );
	 
//Timestamp
reg [38:0] time_count; //39 bit timestamp, 40th bit will be overflow flag

always @ (posedge clk or posedge reset)
begin
	if (reset)
		time_count <= 39'h0;
	else
		time_count <= time_count + 1;
end

//Convert one Hex digit to its ASCII representation
function [7:0] nibbleToASCII;
input [3:0] nibble;
begin
	if (nibble == 4'h0) nibbleToASCII = 8'h30;
	else if (nibble == 4'h1) nibbleToASCII = 8'h31;
	else if (nibble == 4'h2) nibbleToASCII = 8'h32;
	else if (nibble == 4'h3) nibbleToASCII = 8'h33;
	else if (nibble == 4'h4) nibbleToASCII = 8'h34;
	else if (nibble == 4'h5) nibbleToASCII = 8'h35;
	else if (nibble == 4'h6) nibbleToASCII = 8'h36;
	else if (nibble == 4'h7) nibbleToASCII = 8'h37;
	else if (nibble == 4'h8) nibbleToASCII = 8'h38;
	else if (nibble == 4'h9) nibbleToASCII = 8'h39;
	else if (nibble == 4'hA) nibbleToASCII = 8'h41;
	else if (nibble == 4'hB) nibbleToASCII = 8'h42;
	else if (nibble == 4'hC) nibbleToASCII = 8'h43;
	else if (nibble == 4'hD) nibbleToASCII = 8'h44;
	else if (nibble == 4'hE) nibbleToASCII = 8'h45;
	else nibbleToASCII = 8'h46;
end
endfunction
	 
//Create the FIFOs
wire [102:0] rx_fifo_din, rx_fifo_dout;
wire rx_wr_en;
reg rx_rd_en, rx_rd_en_next;
wire rx_overflow, rx_empty;
//serial debug fifo: 103 bits wide, first word fall through
serial_debug_fifo rx_fifo (
  .clk (clk),
  .rst (reset),
  .din (rx_fifo_din),
  .wr_en (rx_wr_en),
  .rd_en (rx_rd_en),
  .dout (rx_fifo_dout),
  .full (),
  .overflow (rx_overflow),
  .empty (rx_empty)
);

assign rx_fifo_din = {time_count, rx_message};
assign rx_wr_en = rx_en;

wire [102:0] tx_fifo_din, tx_fifo_dout;
wire tx_wr_en;
reg tx_rd_en, tx_rd_en_next;
wire tx_overflow, tx_empty;
serial_debug_fifo tx_fifo (
  .clk (clk),
  .rst (reset),
  .din (tx_fifo_din),
  .wr_en (tx_wr_en),
  .rd_en (tx_rd_en),
  .dout (tx_fifo_dout),
  .full (),
  .overflow (tx_overflow),
  .empty (tx_empty)
);

assign tx_fifo_din = {time_count, tx_message};
assign tx_wr_en = tx_en;

wire [102:0] master_fifo_din, master_fifo_dout;
wire master_wr_en;
reg master_rd_en, master_rd_en_next;
wire master_overflow, master_empty;
serial_debug_fifo master_fifo (
  .clk (clk),
  .rst (reset),
  .din (master_fifo_din),
  .wr_en (master_wr_en),
  .rd_en (master_rd_en),
  .dout (master_fifo_dout),
  .full (),
  .overflow (master_overflow),
  .empty (master_empty)
);

assign master_fifo_din = {time_count, master_message};
assign master_wr_en = master_en;

wire [102:0] prim_fifo_din, prim_fifo_dout;
wire prim_wr_en;
reg prim_rd_en, prim_rd_en_next;
wire prim_overflow, prim_empty;
serial_debug_fifo prim_fifo (
  .clk (clk),
  .rst (reset),
  .din (prim_fifo_din),
  .wr_en (prim_wr_en),
  .rd_en (prim_rd_en),
  .dout (prim_fifo_dout),
  .full (),
  .overflow (prim_overflow),
  .empty (prim_empty)
);

assign prim_fifo_din = {time_count, prim_message};
assign prim_wr_en = prim_en;

//Arbiter logic
wire [38:0] rx_time, tx_time, master_time, prim_time;
assign rx_time = (~rx_empty) ? rx_fifo_dout[102:64] : 39'hFFFFFFFFFF;
assign tx_time = (~tx_empty) ? tx_fifo_dout[102:64] : 39'hFFFFFFFFFF;
assign master_time = (~master_empty) ? master_fifo_dout[102:64] : 39'hFFFFFFFFFF; 
assign prim_time = (~prim_empty) ? prim_fifo_dout[102:64] : 39'hFFFFFFFFFF;

reg [127:0] message_next;
reg message_wr_en_next;
reg [39:0] message_time; //the timestamp that will be written with the message
reg [63:0] message_body;

always @ (*)
begin
	message_wr_en_next = 1'b0;
	message_time = 40'hDEADDEADDE;
	message_body = 64'hBEEFBEEFBEEFBEEF;
	master_rd_en_next = 1'b0;
	tx_rd_en_next = 1'b0;
	rx_rd_en_next = 1'b0;
	prim_rd_en_next = 1'b0;
	
	if (~message_fifo_full) begin
		//Highest priority: master
		if ((master_time <= rx_time) && (master_time <= tx_time) && (master_time <= prim_time)) begin
			if (~master_empty) begin
				message_time = {master_overflow, master_time};
				message_body = master_fifo_dout[63:0];
				message_wr_en_next = 1'b1;
				master_rd_en_next = 1'b1;
			end
		end
		else if ((tx_time <= rx_time) && (tx_time <= prim_time)) begin
			if (~tx_empty) begin
				message_time = {tx_overflow, tx_time};
				message_body = tx_fifo_dout[63:0];
				message_wr_en_next = 1'b1;
				tx_rd_en_next = 1'b1;
			end
		end
		else if (rx_time <= prim_time) begin
			if (~rx_empty) begin
				message_time = {rx_overflow, rx_time};
				message_body = rx_fifo_dout[63:0];
				message_wr_en_next = 1'b1;
				rx_rd_en_next = 1'b1;
			end
		end
		else if (~prim_empty) begin
			message_time = {prim_overflow, prim_time};
			message_body = prim_fifo_dout[63:0];
			message_wr_en_next = 1'b1;
			prim_rd_en_next = 1'b1;
		end
	end
	
	//Convert timestamp to ASCII
	message_next[127:120] = nibbleToASCII (message_time[23:20]);
	message_next[119:112] = nibbleToASCII (message_time[19:16]);
	message_next[111:104] = nibbleToASCII (message_time[15:12]);
	message_next[103: 96] = nibbleToASCII (message_time[11: 8]);
	message_next[ 95: 88] = nibbleToASCII (message_time[ 7: 4]);
	message_next[ 87: 80] = nibbleToASCII (message_time[ 3: 0]);
	//Separator
   message_next[79:72] = ":";
	//Message
	message_next[ 71:  8] = message_body;
	//Carriage Return
	message_next[  7:  0] = 8'h0D;
end
	
				

always @ (posedge clk or posedge reset)
begin
	if (reset) begin
		message <= 128'h0;
		message_wr_en <= 1'b0;
		master_rd_en <= 1'b0;
		tx_rd_en <= 1'b0;
		rx_rd_en <= 1'b0;
		prim_rd_en <= 1'b0;
	end
	else begin
		message <= message_next;
		message_wr_en <= message_wr_en_next;
		master_rd_en <= master_rd_en_next;
		tx_rd_en <= tx_rd_en_next;
		rx_rd_en <= rx_rd_en_next;
		prim_rd_en <= prim_rd_en_next;
	end
end


endmodule
