`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name:    sata_core
// Module Name:    serial_top 
// Project Name: 
// Target Devices: Virtex4
// Tool versions: ISE 14.2
// Description: Receives messages from the monitors and places them in an arbiter.
// Puts the output of the arbiter in a FIFO, and then interfaces the FIFO with a
// UART core.
//
// Be sure to change the UART's CLOCK_DIVIDE parameter if using a different sys_clk
// speed. Default: 100MHz sys_clk, 28800 baud.
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module serial_top(
		input sys_clk,
		input sata_clk,
		input reset,
		input serial_rx,
		output serial_tx,
		input [63:0] rx_message,
		input rx_en,
		input [63:0] tx_message,
		input tx_en,
		input [63:0] master_message,
		input master_en,
		input [63:0] prim_message,
		input prim_en
    );
	 
wire message_wr_en;

reg message_rd_en, message_rd_en_next;
wire [127:0] message_dout, message_din;
wire message_empty, message_full;

reg [127:0] message_data, message_data_next;
reg [7:0] tx_byte, tx_byte_next;
reg uart_transmit, uart_transmit_next;
wire uart_not_ready;


serial_arbiter serial_arbiter_i (
    .clk (sata_clk),
    .reset (reset),
    .rx_message (rx_message),
    .rx_en (rx_en),
    .tx_message (tx_message),
    .tx_en (tx_en),
    .master_message (master_message),
    .master_en (master_en),
    .prim_message (prim_message),
    .prim_en (prim_en),
	 .message (message_din),
	 .message_wr_en (message_wr_en),
	 .message_fifo_full (message_full)
    );

//message fifo: 128 bits wide, first word fall through, 16 deep
//TODO fix this
message_fifo message_fifo_i (
  .wr_clk (sata_clk),
  .rd_clk (sys_clk),
  .rst (reset),
  .din (message_din),
  .wr_en (message_wr_en),
  .rd_en (message_rd_en),
  .dout (message_dout),
  .full (message_full),
  .almost_empty (message_empty),
  .empty()
);

uart #(.CLOCK_DIVIDE (868)) uart_i ( //28800 baud
    .clk (sys_clk), // The master clock for this module
    .rst (reset), // Synchronous reset.
    .rx (serial_rx), // Incoming serial line
    .tx (serial_tx), // Outgoing serial line
    .transmit (uart_transmit), // Signal to transmit
    .tx_byte (tx_byte), // Byte to transmit
    .received (), // Indicated that a byte has been received.
    .rx_byte (), // Byte received
    .is_receiving (), // Low when receive line is idle.
    .is_transmitting (uart_not_ready), // Low when transmit line is idle.
    .recv_error () // Indicates error in receiving packet.
);

reg [1:0] state, state_next;
reg [3:0] count, count_next;
parameter [1:0] s_idle     = 2'b00,
					 s_transmit = 2'b01,
					 s_shift		= 2'b10;

always @ (*)
begin
	message_rd_en_next = 1'b0;
	message_data_next = message_data;
	tx_byte_next = tx_byte;
	uart_transmit_next = 1'b0;
	state_next = state;
	count_next = count;
	
	case (state)
	//Wait for a message to be ready.
	
		s_idle: begin
			count_next = 4'b0;
			//grab a message if there is one ready for us
			if (~message_empty) begin
				message_rd_en_next = 1'b1;
				message_data_next = message_dout;
				state_next = s_transmit;
			end
		end
		
	//Transmit 1 character of the message.
		s_transmit: begin
			tx_byte_next = message_data[127:120];
			if (~uart_not_ready) begin
				uart_transmit_next = 1'b1;
				if (count == 4'hF) begin //end of message reached
					message_data_next = 128'h0;
					state_next = s_idle;
				end
				else
					state_next = s_shift;
			end
		end
		
	//Shift to the next character in the message.
		s_shift: begin
			message_data_next = message_data << 8; //next character
			count_next = count + 1;
			state_next = s_transmit;
		end
	endcase
end

always @ (posedge sys_clk or posedge reset)
begin
	if (reset) begin
		message_rd_en <= 1'b0;
		message_data <= 128'h0;
		tx_byte <= 8'b0;
		uart_transmit <= 1'b0;
		state <= s_idle;
		count <= 4'b0;
	end
	else begin
		message_rd_en <= message_rd_en_next;
		message_data <= message_data_next;
		tx_byte <= tx_byte_next;
		uart_transmit <= uart_transmit_next;
		state <= state_next;
		count <= count_next;
	end
end


endmodule
